#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import ChatPermissions, User
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def chat_permissions():
    return ChatPermissions(
        can_send_messages=True,
        can_send_polls=True,
        can_send_other_messages=True,
        can_add_web_page_previews=True,
        can_change_info=True,
        can_invite_users=True,
        can_pin_messages=True,
        can_manage_topics=True,
        can_send_audios=True,
        can_send_documents=True,
        can_send_photos=True,
        can_send_videos=True,
        can_send_video_notes=True,
        can_send_voice_notes=True,
        can_edit_tag=True,
    )


class ChatPermissionsTestBase:
    can_send_messages = True
    can_send_polls = True
    can_send_other_messages = False
    can_add_web_page_previews = False
    can_change_info = False
    can_invite_users = None
    can_pin_messages = None
    can_manage_topics = None
    can_send_audios = True
    can_send_documents = False
    can_send_photos = None
    can_send_videos = True
    can_send_video_notes = False
    can_send_voice_notes = None
    can_edit_tag = None


class TestChatPermissionsWithoutRequest(ChatPermissionsTestBase):
    def test_slot_behaviour(self, chat_permissions):
        inst = chat_permissions
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "can_send_messages": self.can_send_messages,
            "can_send_media_messages": "can_send_media_messages",
            "can_send_polls": self.can_send_polls,
            "can_send_other_messages": self.can_send_other_messages,
            "can_add_web_page_previews": self.can_add_web_page_previews,
            "can_change_info": self.can_change_info,
            "can_invite_users": self.can_invite_users,
            "can_pin_messages": self.can_pin_messages,
            "can_send_audios": self.can_send_audios,
            "can_send_documents": self.can_send_documents,
            "can_send_photos": self.can_send_photos,
            "can_send_videos": self.can_send_videos,
            "can_send_video_notes": self.can_send_video_notes,
            "can_send_voice_notes": self.can_send_voice_notes,
            "can_edit_tag": self.can_edit_tag,
        }
        permissions = ChatPermissions.de_json(json_dict, offline_bot)
        assert permissions.api_kwargs == {"can_send_media_messages": "can_send_media_messages"}

        assert permissions.can_send_messages == self.can_send_messages
        assert permissions.can_send_polls == self.can_send_polls
        assert permissions.can_send_other_messages == self.can_send_other_messages
        assert permissions.can_add_web_page_previews == self.can_add_web_page_previews
        assert permissions.can_change_info == self.can_change_info
        assert permissions.can_invite_users == self.can_invite_users
        assert permissions.can_pin_messages == self.can_pin_messages
        assert permissions.can_manage_topics == self.can_manage_topics
        assert permissions.can_send_audios == self.can_send_audios
        assert permissions.can_send_documents == self.can_send_documents
        assert permissions.can_send_photos == self.can_send_photos
        assert permissions.can_send_videos == self.can_send_videos
        assert permissions.can_send_video_notes == self.can_send_video_notes
        assert permissions.can_send_voice_notes == self.can_send_voice_notes
        assert permissions.can_edit_tag == self.can_edit_tag

    def test_to_dict(self, chat_permissions):
        permissions_dict = chat_permissions.to_dict()

        assert isinstance(permissions_dict, dict)
        assert permissions_dict["can_send_messages"] == chat_permissions.can_send_messages
        assert permissions_dict["can_send_polls"] == chat_permissions.can_send_polls
        assert (
            permissions_dict["can_send_other_messages"] == chat_permissions.can_send_other_messages
        )
        assert (
            permissions_dict["can_add_web_page_previews"]
            == chat_permissions.can_add_web_page_previews
        )
        assert permissions_dict["can_change_info"] == chat_permissions.can_change_info
        assert permissions_dict["can_invite_users"] == chat_permissions.can_invite_users
        assert permissions_dict["can_pin_messages"] == chat_permissions.can_pin_messages
        assert permissions_dict["can_manage_topics"] == chat_permissions.can_manage_topics
        assert permissions_dict["can_send_audios"] == chat_permissions.can_send_audios
        assert permissions_dict["can_send_documents"] == chat_permissions.can_send_documents
        assert permissions_dict["can_send_photos"] == chat_permissions.can_send_photos
        assert permissions_dict["can_send_videos"] == chat_permissions.can_send_videos
        assert permissions_dict["can_send_video_notes"] == chat_permissions.can_send_video_notes
        assert permissions_dict["can_send_voice_notes"] == chat_permissions.can_send_voice_notes
        assert permissions_dict["can_edit_tag"] == chat_permissions.can_edit_tag

    def test_equality(self):
        a = ChatPermissions(
            can_send_messages=True,
            can_send_polls=True,
            can_send_other_messages=False,
        )
        b = ChatPermissions(
            can_send_polls=True,
            can_send_other_messages=False,
            can_send_messages=True,
        )
        c = ChatPermissions(
            can_send_messages=False,
            can_send_polls=True,
            can_send_other_messages=False,
        )
        d = User(123, "", False)
        e = ChatPermissions(
            can_send_messages=True,
            can_send_polls=True,
            can_send_other_messages=False,
            can_send_audios=True,
            can_send_documents=True,
            can_send_photos=True,
            can_send_videos=True,
            can_send_video_notes=True,
            can_send_voice_notes=True,
            can_edit_tag=True,
        )
        f = ChatPermissions(
            can_send_messages=True,
            can_send_polls=True,
            can_send_other_messages=False,
            can_send_audios=True,
            can_send_documents=True,
            can_send_photos=True,
            can_send_videos=True,
            can_send_video_notes=True,
            can_send_voice_notes=True,
            can_edit_tag=True,
        )

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert e == f
        assert hash(e) == hash(f)

    def test_all_permissions(self):
        f = ChatPermissions()
        t = ChatPermissions.all_permissions()
        # if the dirs are the same, the attributes will all be there
        assert dir(f) == dir(t)
        # now we just need to check that all attributes are True. _id_attrs returns all values,
        # if a new one is added without defaulting to True, this will fail
        for key in t.__slots__:
            assert t[key] is True
        # and as a finisher, make sure the default is different.
        assert f != t

    def test_no_permissions(self):
        f = ChatPermissions()
        t = ChatPermissions.no_permissions()
        # if the dirs are the same, the attributes will all be there
        assert dir(f) == dir(t)
        # now we just need to check that all attributes are True. _id_attrs returns all values,
        # if a new one is added without defaulting to False, this will fail
        for key in t.__slots__:
            assert t[key] is False
        # and as a finisher, make sure the default is different.
        assert f != t



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import datetime as dtm

import pytest

from telegram import (
    BotCommand,
    Chat,
    Giveaway,
    GiveawayCompleted,
    GiveawayCreated,
    GiveawayWinners,
    Message,
    User,
)
from telegram._utils.datetime import UTC, to_timestamp
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def giveaway():
    return Giveaway(
        chats=[Chat(1, Chat.CHANNEL), Chat(2, Chat.SUPERGROUP)],
        winners_selection_date=TestGiveawayWithoutRequest.winners_selection_date,
        winner_count=TestGiveawayWithoutRequest.winner_count,
        only_new_members=TestGiveawayWithoutRequest.only_new_members,
        has_public_winners=TestGiveawayWithoutRequest.has_public_winners,
        prize_description=TestGiveawayWithoutRequest.prize_description,
        country_codes=TestGiveawayWithoutRequest.country_codes,
        premium_subscription_month_count=(
            TestGiveawayWithoutRequest.premium_subscription_month_count
        ),
        prize_star_count=TestGiveawayWithoutRequest.prize_star_count,
    )


class TestGiveawayWithoutRequest:
    chats = [Chat(1, Chat.CHANNEL), Chat(2, Chat.SUPERGROUP)]
    winners_selection_date = dtm.datetime.now(dtm.timezone.utc).replace(microsecond=0)
    winner_count = 42
    only_new_members = True
    has_public_winners = True
    prize_description = "prize_description"
    country_codes = ["DE", "US"]
    premium_subscription_month_count = 3
    prize_star_count = 99

    def test_slot_behaviour(self, giveaway):
        for attr in giveaway.__slots__:
            assert getattr(giveaway, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(giveaway)) == len(set(mro_slots(giveaway))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "chats": [chat.to_dict() for chat in self.chats],
            "winners_selection_date": to_timestamp(self.winners_selection_date),
            "winner_count": self.winner_count,
            "only_new_members": self.only_new_members,
            "has_public_winners": self.has_public_winners,
            "prize_description": self.prize_description,
            "country_codes": self.country_codes,
            "premium_subscription_month_count": self.premium_subscription_month_count,
            "prize_star_count": self.prize_star_count,
        }

        giveaway = Giveaway.de_json(json_dict, offline_bot)
        assert giveaway.api_kwargs == {}

        assert giveaway.chats == tuple(self.chats)
        assert giveaway.winners_selection_date == self.winners_selection_date
        assert giveaway.winner_count == self.winner_count
        assert giveaway.only_new_members == self.only_new_members
        assert giveaway.has_public_winners == self.has_public_winners
        assert giveaway.prize_description == self.prize_description
        assert giveaway.country_codes == tuple(self.country_codes)
        assert giveaway.premium_subscription_month_count == self.premium_subscription_month_count
        assert giveaway.prize_star_count == self.prize_star_count

    def test_de_json_localization(self, tz_bot, offline_bot, raw_bot):
        json_dict = {
            "chats": [chat.to_dict() for chat in self.chats],
            "winners_selection_date": to_timestamp(self.winners_selection_date),
            "winner_count": self.winner_count,
            "only_new_members": self.only_new_members,
            "has_public_winners": self.has_public_winners,
            "prize_description": self.prize_description,
            "country_codes": self.country_codes,
            "premium_subscription_month_count": self.premium_subscription_month_count,
            "prize_star_count": self.prize_star_count,
        }

        giveaway_raw = Giveaway.de_json(json_dict, raw_bot)
        giveaway_bot = Giveaway.de_json(json_dict, offline_bot)
        giveaway_bot_tz = Giveaway.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        giveaway_bot_tz_offset = giveaway_bot_tz.winners_selection_date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
            giveaway_bot_tz.winners_selection_date.replace(tzinfo=None)
        )

        assert giveaway_raw.winners_selection_date.tzinfo == UTC
        assert giveaway_bot.winners_selection_date.tzinfo == UTC
        assert giveaway_bot_tz_offset == tz_bot_offset

    def test_to_dict(self, giveaway):
        giveaway_dict = giveaway.to_dict()

        assert isinstance(giveaway_dict, dict)
        assert giveaway_dict["chats"] == [chat.to_dict() for chat in self.chats]
        assert giveaway_dict["winners_selection_date"] == to_timestamp(self.winners_selection_date)
        assert giveaway_dict["winner_count"] == self.winner_count
        assert giveaway_dict["only_new_members"] == self.only_new_members
        assert giveaway_dict["has_public_winners"] == self.has_public_winners
        assert giveaway_dict["prize_description"] == self.prize_description
        assert giveaway_dict["country_codes"] == self.country_codes
        assert (
            giveaway_dict["premium_subscription_month_count"]
            == self.premium_subscription_month_count
        )
        assert giveaway_dict["prize_star_count"] == self.prize_star_count

    def test_equality(self, giveaway):
        a = giveaway
        b = Giveaway(
            chats=self.chats,
            winners_selection_date=self.winners_selection_date,
            winner_count=self.winner_count,
        )
        c = Giveaway(
            chats=self.chats,
            winners_selection_date=self.winners_selection_date + dtm.timedelta(seconds=100),
            winner_count=self.winner_count,
        )
        d = Giveaway(
            chats=self.chats, winners_selection_date=self.winners_selection_date, winner_count=17
        )
        e = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def giveaway_created():
    return GiveawayCreated(
        prize_star_count=TestGiveawayCreatedWithoutRequest.prize_star_count,
    )


class TestGiveawayCreatedWithoutRequest:
    prize_star_count = 99

    def test_slot_behaviour(self, giveaway_created):
        for attr in giveaway_created.__slots__:
            assert getattr(giveaway_created, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(giveaway_created)) == len(set(mro_slots(giveaway_created))), (
            "duplicate slot"
        )

    def test_de_json(self, bot):
        json_dict = {
            "prize_star_count": self.prize_star_count,
        }

        gac = GiveawayCreated.de_json(json_dict, bot)
        assert gac.api_kwargs == {}
        assert gac.prize_star_count == self.prize_star_count

    def test_to_dict(self, giveaway_created):
        gac_dict = giveaway_created.to_dict()

        assert isinstance(gac_dict, dict)
        assert gac_dict["prize_star_count"] == self.prize_star_count


@pytest.fixture(scope="module")
def giveaway_winners():
    return GiveawayWinners(
        chat=TestGiveawayWinnersWithoutRequest.chat,
        giveaway_message_id=TestGiveawayWinnersWithoutRequest.giveaway_message_id,
        winners_selection_date=TestGiveawayWinnersWithoutRequest.winners_selection_date,
        winner_count=TestGiveawayWinnersWithoutRequest.winner_count,
        winners=TestGiveawayWinnersWithoutRequest.winners,
        only_new_members=TestGiveawayWinnersWithoutRequest.only_new_members,
        prize_description=TestGiveawayWinnersWithoutRequest.prize_description,
        premium_subscription_month_count=(
            TestGiveawayWinnersWithoutRequest.premium_subscription_month_count
        ),
        additional_chat_count=TestGiveawayWinnersWithoutRequest.additional_chat_count,
        unclaimed_prize_count=TestGiveawayWinnersWithoutRequest.unclaimed_prize_count,
        was_refunded=TestGiveawayWinnersWithoutRequest.was_refunded,
        prize_star_count=TestGiveawayWinnersWithoutRequest.prize_star_count,
    )


class TestGiveawayWinnersWithoutRequest:
    chat = Chat(1, Chat.CHANNEL)
    giveaway_message_id = 123456789
    winners_selection_date = dtm.datetime.now(dtm.timezone.utc).replace(microsecond=0)
    winner_count = 42
    winners = [User(1, "user1", False), User(2, "user2", False)]
    additional_chat_count = 2
    premium_subscription_month_count = 3
    unclaimed_prize_count = 4
    only_new_members = True
    was_refunded = True
    prize_description = "prize_description"
    prize_star_count = 99

    def test_slot_behaviour(self, giveaway_winners):
        for attr in giveaway_winners.__slots__:
            assert getattr(giveaway_winners, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(giveaway_winners)) == len(set(mro_slots(giveaway_winners))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "chat": self.chat.to_dict(),
            "giveaway_message_id": self.giveaway_message_id,
            "winners_selection_date": to_timestamp(self.winners_selection_date),
            "winner_count": self.winner_count,
            "winners": [winner.to_dict() for winner in self.winners],
            "additional_chat_count": self.additional_chat_count,
            "premium_subscription_month_count": self.premium_subscription_month_count,
            "unclaimed_prize_count": self.unclaimed_prize_count,
            "only_new_members": self.only_new_members,
            "was_refunded": self.was_refunded,
            "prize_description": self.prize_description,
            "prize_star_count": self.prize_star_count,
        }

        giveaway_winners = GiveawayWinners.de_json(json_dict, offline_bot)
        assert giveaway_winners.api_kwargs == {}

        assert giveaway_winners.chat == self.chat
        assert giveaway_winners.giveaway_message_id == self.giveaway_message_id
        assert giveaway_winners.winners_selection_date == self.winners_selection_date
        assert giveaway_winners.winner_count == self.winner_count
        assert giveaway_winners.winners == tuple(self.winners)
        assert giveaway_winners.additional_chat_count == self.additional_chat_count
        assert (
            giveaway_winners.premium_subscription_month_count
            == self.premium_subscription_month_count
        )
        assert giveaway_winners.unclaimed_prize_count == self.unclaimed_prize_count
        assert giveaway_winners.only_new_members == self.only_new_members
        assert giveaway_winners.was_refunded == self.was_refunded
        assert giveaway_winners.prize_description == self.prize_description
        assert giveaway_winners.prize_star_count == self.prize_star_count

    def test_de_json_localization(self, tz_bot, offline_bot, raw_bot):
        json_dict = {
            "chat": self.chat.to_dict(),
            "giveaway_message_id": self.giveaway_message_id,
            "winners_selection_date": to_timestamp(self.winners_selection_date),
            "winner_count": self.winner_count,
            "winners": [winner.to_dict() for winner in self.winners],
        }

        giveaway_winners_raw = GiveawayWinners.de_json(json_dict, raw_bot)
        giveaway_winners_bot = GiveawayWinners.de_json(json_dict, offline_bot)
        giveaway_winners_bot_tz = GiveawayWinners.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        giveaway_winners_bot_tz_offset = giveaway_winners_bot_tz.winners_selection_date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
            giveaway_winners_bot_tz.winners_selection_date.replace(tzinfo=None)
        )

        assert giveaway_winners_raw.winners_selection_date.tzinfo == UTC
        assert giveaway_winners_bot.winners_selection_date.tzinfo == UTC
        assert giveaway_winners_bot_tz_offset == tz_bot_offset

    def test_to_dict(self, giveaway_winners):
        giveaway_winners_dict = giveaway_winners.to_dict()

        assert isinstance(giveaway_winners_dict, dict)
        assert giveaway_winners_dict["chat"] == self.chat.to_dict()
        assert giveaway_winners_dict["giveaway_message_id"] == self.giveaway_message_id
        assert giveaway_winners_dict["winners_selection_date"] == to_timestamp(
            self.winners_selection_date
        )
        assert giveaway_winners_dict["winner_count"] == self.winner_count
        assert giveaway_winners_dict["winners"] == [winner.to_dict() for winner in self.winners]
        assert giveaway_winners_dict["additional_chat_count"] == self.additional_chat_count
        assert (
            giveaway_winners_dict["premium_subscription_month_count"]
            == self.premium_subscription_month_count
        )
        assert giveaway_winners_dict["unclaimed_prize_count"] == self.unclaimed_prize_count
        assert giveaway_winners_dict["only_new_members"] == self.only_new_members
        assert giveaway_winners_dict["was_refunded"] == self.was_refunded
        assert giveaway_winners_dict["prize_description"] == self.prize_description
        assert giveaway_winners_dict["prize_star_count"] == self.prize_star_count

    def test_equality(self, giveaway_winners):
        a = giveaway_winners
        b = GiveawayWinners(
            chat=self.chat,
            giveaway_message_id=self.giveaway_message_id,
            winners_selection_date=self.winners_selection_date,
            winner_count=self.winner_count,
            winners=self.winners,
        )
        c = GiveawayWinners(
            chat=self.chat,
            giveaway_message_id=self.giveaway_message_id,
            winners_selection_date=self.winners_selection_date + dtm.timedelta(seconds=100),
            winner_count=self.winner_count,
            winners=self.winners,
        )
        d = GiveawayWinners(
            chat=self.chat,
            giveaway_message_id=self.giveaway_message_id,
            winners_selection_date=self.winners_selection_date,
            winner_count=17,
            winners=self.winners,
        )
        e = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def giveaway_completed():
    return GiveawayCompleted(
        winner_count=TestGiveawayCompletedWithoutRequest.winner_count,
        unclaimed_prize_count=TestGiveawayCompletedWithoutRequest.unclaimed_prize_count,
        giveaway_message=TestGiveawayCompletedWithoutRequest.giveaway_message,
        is_star_giveaway=TestGiveawayCompletedWithoutRequest.is_star_giveaway,
    )


class TestGiveawayCompletedWithoutRequest:
    winner_count = 42
    unclaimed_prize_count = 4
    is_star_giveaway = True
    giveaway_message = Message(
        message_id=1,
        date=dtm.datetime.now(dtm.timezone.utc),
        text="giveaway_message",
        chat=Chat(1, Chat.CHANNEL),
        from_user=User(1, "user1", False),
    )

    def test_slot_behaviour(self, giveaway_completed):
        for attr in giveaway_completed.__slots__:
            assert getattr(giveaway_completed, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(giveaway_completed)) == len(set(mro_slots(giveaway_completed))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "winner_count": self.winner_count,
            "unclaimed_prize_count": self.unclaimed_prize_count,
            "giveaway_message": self.giveaway_message.to_dict(),
            "is_star_giveaway": self.is_star_giveaway,
        }

        giveaway_completed = GiveawayCompleted.de_json(json_dict, offline_bot)
        assert giveaway_completed.api_kwargs == {}

        assert giveaway_completed.winner_count == self.winner_count
        assert giveaway_completed.unclaimed_prize_count == self.unclaimed_prize_count
        assert giveaway_completed.giveaway_message == self.giveaway_message
        assert giveaway_completed.is_star_giveaway == self.is_star_giveaway

    def test_to_dict(self, giveaway_completed):
        giveaway_completed_dict = giveaway_completed.to_dict()

        assert isinstance(giveaway_completed_dict, dict)
        assert giveaway_completed_dict["winner_count"] == self.winner_count
        assert giveaway_completed_dict["unclaimed_prize_count"] == self.unclaimed_prize_count
        assert giveaway_completed_dict["giveaway_message"] == self.giveaway_message.to_dict()
        assert giveaway_completed_dict["is_star_giveaway"] == self.is_star_giveaway

    def test_equality(self, giveaway_completed):
        a = giveaway_completed
        b = GiveawayCompleted(
            winner_count=self.winner_count,
            unclaimed_prize_count=self.unclaimed_prize_count,
            giveaway_message=self.giveaway_message,
            is_star_giveaway=self.is_star_giveaway,
        )
        c = GiveawayCompleted(
            winner_count=self.winner_count + 30,
            unclaimed_prize_count=self.unclaimed_prize_count,
        )
        d = GiveawayCompleted(
            winner_count=self.winner_count,
            unclaimed_prize_count=17,
            giveaway_message=self.giveaway_message,
        )
        e = GiveawayCompleted(
            winner_count=self.winner_count + 1,
            unclaimed_prize_count=self.unclaimed_prize_count,
        )
        f = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert a != f
        assert hash(a) != hash(f)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the TestDirectMessagesTopic class."""

import pytest

from telegram import DirectMessagesTopic, User
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def direct_messages_topic(offline_bot):
    dmt = DirectMessagesTopic(
        topic_id=DirectMessagesTopicTestBase.topic_id,
        user=DirectMessagesTopicTestBase.user,
    )
    dmt.set_bot(offline_bot)
    dmt._unfreeze()
    return dmt


class DirectMessagesTopicTestBase:
    topic_id = 12345
    user = User(id=67890, is_bot=False, first_name="Test")


class TestDirectMessagesTopicWithoutRequest(DirectMessagesTopicTestBase):
    def test_slot_behaviour(self, direct_messages_topic):
        cfi = direct_messages_topic
        for attr in cfi.__slots__:
            assert getattr(cfi, attr, "err") != "err", f"got extra slot '{attr}'"

        assert len(mro_slots(cfi)) == len(set(mro_slots(cfi))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "topic_id": self.topic_id,
            "user": self.user.to_dict(),
        }

        dmt = DirectMessagesTopic.de_json(json_dict, offline_bot)
        assert dmt.topic_id == self.topic_id
        assert dmt.user == self.user
        assert dmt.api_kwargs == {}

    def test_to_dict(self, direct_messages_topic):
        dmt = direct_messages_topic
        dmt_dict = dmt.to_dict()

        assert isinstance(dmt_dict, dict)
        assert dmt_dict["topic_id"] == dmt.topic_id
        assert dmt_dict["user"] == dmt.user.to_dict()

    def test_equality(self, direct_messages_topic):
        dmt_1 = direct_messages_topic
        dmt_2 = DirectMessagesTopic(
            topic_id=dmt_1.topic_id,
            user=dmt_1.user,
        )
        assert dmt_1 == dmt_2
        assert hash(dmt_1) == hash(dmt_2)

        random = User(id=99999, is_bot=False, first_name="Random")
        assert random != dmt_2
        assert hash(random) != hash(dmt_2)

        dmt_3 = DirectMessagesTopic(
            topic_id=8371,
            user=dmt_1.user,
        )
        assert dmt_1 != dmt_3
        assert hash(dmt_1) != hash(dmt_3)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm
import inspect
from copy import deepcopy

import pytest

from telegram import (
    Chat,
    Dice,
    MessageOrigin,
    MessageOriginChannel,
    MessageOriginChat,
    MessageOriginHiddenUser,
    MessageOriginUser,
    User,
)
from telegram._utils.datetime import UTC, to_timestamp
from tests.auxil.slots import mro_slots

ignored = ["self", "api_kwargs"]


class MODefaults:
    date: dtm.datetime = to_timestamp(dtm.datetime.utcnow())
    chat = Chat(1, Chat.CHANNEL)
    message_id = 123
    author_signautre = "PTB"
    sender_chat = Chat(1, Chat.CHANNEL)
    sender_user_name = "PTB"
    sender_user = User(1, "user", False)


def message_origin_channel():
    return MessageOriginChannel(
        MODefaults.date, MODefaults.chat, MODefaults.message_id, MODefaults.author_signautre
    )


def message_origin_chat():
    return MessageOriginChat(
        MODefaults.date,
        MODefaults.sender_chat,
        MODefaults.author_signautre,
    )


def message_origin_hidden_user():
    return MessageOriginHiddenUser(MODefaults.date, MODefaults.sender_user_name)


def message_origin_user():
    return MessageOriginUser(MODefaults.date, MODefaults.sender_user)


def make_json_dict(instance: MessageOrigin, include_optional_args: bool = False) -> dict:
    """Used to make the json dict which we use for testing de_json. Similar to iter_args()"""
    json_dict = {"type": instance.type}
    sig = inspect.signature(instance.__class__.__init__)

    for param in sig.parameters.values():
        if param.name in ignored:  # ignore irrelevant params
            continue

        val = getattr(instance, param.name)
        # Compulsory args-
        if param.default is inspect.Parameter.empty:
            if hasattr(val, "to_dict"):  # convert the user object or any future ones to dict.
                val = val.to_dict()
            json_dict[param.name] = val

        # If we want to test all args (for de_json)-
        elif param.default is not inspect.Parameter.empty and include_optional_args:
            json_dict[param.name] = val
    return json_dict


def iter_args(
    instance: MessageOrigin, de_json_inst: MessageOrigin, include_optional: bool = False
):
    """
    We accept both the regular instance and de_json created instance and iterate over them for
    easy one line testing later one.
    """
    yield instance.type, de_json_inst.type  # yield this here cause it's not available in sig.

    sig = inspect.signature(instance.__class__.__init__)
    for param in sig.parameters.values():
        if param.name in ignored:
            continue
        inst_at, json_at = getattr(instance, param.name), getattr(de_json_inst, param.name)
        if isinstance(json_at, dtm.datetime):  # Convert datetime to int
            json_at = to_timestamp(json_at)
        if (
            param.default is not inspect.Parameter.empty and include_optional
        ) or param.default is inspect.Parameter.empty:
            yield inst_at, json_at


@pytest.fixture
def message_origin_type(request):
    return request.param()


@pytest.mark.parametrize(
    "message_origin_type",
    [
        message_origin_channel,
        message_origin_chat,
        message_origin_hidden_user,
        message_origin_user,
    ],
    indirect=True,
)
class TestMessageOriginTypesWithoutRequest:
    def test_slot_behaviour(self, message_origin_type):
        inst = message_origin_type
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json_required_args(self, offline_bot, message_origin_type):
        cls = message_origin_type.__class__

        json_dict = make_json_dict(message_origin_type)
        const_message_origin = MessageOrigin.de_json(json_dict, offline_bot)
        assert const_message_origin.api_kwargs == {}

        assert isinstance(const_message_origin, MessageOrigin)
        assert isinstance(const_message_origin, cls)
        for msg_origin_type_at, const_msg_origin_at in iter_args(
            message_origin_type, const_message_origin
        ):
            assert msg_origin_type_at == const_msg_origin_at

    def test_de_json_all_args(self, offline_bot, message_origin_type):
        json_dict = make_json_dict(message_origin_type, include_optional_args=True)
        const_message_origin = MessageOrigin.de_json(json_dict, offline_bot)

        assert const_message_origin.api_kwargs == {}

        assert isinstance(const_message_origin, MessageOrigin)
        assert isinstance(const_message_origin, message_origin_type.__class__)
        for msg_origin_type_at, const_msg_origin_at in iter_args(
            message_origin_type, const_message_origin, True
        ):
            assert msg_origin_type_at == const_msg_origin_at

    def test_de_json_messageorigin_localization(
        self, message_origin_type, tz_bot, offline_bot, raw_bot
    ):
        json_dict = make_json_dict(message_origin_type, include_optional_args=True)
        msgorigin_raw = MessageOrigin.de_json(json_dict, raw_bot)
        msgorigin_bot = MessageOrigin.de_json(json_dict, offline_bot)
        msgorigin_tz = MessageOrigin.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        msgorigin_offset = msgorigin_tz.date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(msgorigin_tz.date.replace(tzinfo=None))

        assert msgorigin_raw.date.tzinfo == UTC
        assert msgorigin_bot.date.tzinfo == UTC
        assert msgorigin_offset == tz_bot_offset

    def test_de_json_invalid_type(self, message_origin_type, offline_bot):
        json_dict = {"type": "invalid", "date": MODefaults.date}
        message_origin_type = MessageOrigin.de_json(json_dict, offline_bot)

        assert type(message_origin_type) is MessageOrigin
        assert message_origin_type.type == "invalid"

    def test_de_json_subclass(self, message_origin_type, offline_bot, chat_id):
        """This makes sure that e.g. MessageOriginChat(data, offline_bot) never returns a
        MessageOriginUser instance."""
        cls = message_origin_type.__class__
        json_dict = make_json_dict(message_origin_type, True)
        assert type(cls.de_json(json_dict, offline_bot)) is cls

    def test_to_dict(self, message_origin_type):
        message_origin_dict = message_origin_type.to_dict()

        assert isinstance(message_origin_dict, dict)
        assert message_origin_dict["type"] == message_origin_type.type
        assert message_origin_dict["date"] == message_origin_type.date

        for slot in message_origin_type.__slots__:  # additional verification for the optional args
            if slot in ("chat", "sender_chat", "sender_user"):
                assert (getattr(message_origin_type, slot)).to_dict() == message_origin_dict[slot]
                continue
            assert getattr(message_origin_type, slot) == message_origin_dict[slot]

    def test_equality(self, message_origin_type):
        a = MessageOrigin(type="type", date=MODefaults.date)
        b = MessageOrigin(type="type", date=MODefaults.date)
        c = message_origin_type
        d = deepcopy(message_origin_type)
        e = Dice(4, "emoji")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert c == d
        assert hash(c) == hash(d)

        assert c != e
        assert hash(c) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program. If not, see [http://www.gnu.org/licenses/].
import datetime as dtm

import pytest

from telegram import (
    AffiliateInfo,
    Chat,
    Gift,
    PaidMediaVideo,
    RevenueWithdrawalStatePending,
    Sticker,
    TransactionPartner,
    TransactionPartnerAffiliateProgram,
    TransactionPartnerChat,
    TransactionPartnerFragment,
    TransactionPartnerOther,
    TransactionPartnerTelegramAds,
    TransactionPartnerTelegramApi,
    TransactionPartnerUser,
    User,
    Video,
)
from telegram.constants import TransactionPartnerType
from tests.auxil.slots import mro_slots


@pytest.fixture
def transaction_partner():
    return TransactionPartner(TransactionPartnerTestBase.type)


class TransactionPartnerTestBase:
    type = TransactionPartnerType.AFFILIATE_PROGRAM
    commission_per_mille = 42
    sponsor_user = User(
        id=1,
        is_bot=False,
        first_name="sponsor",
        last_name="user",
    )
    withdrawal_state = RevenueWithdrawalStatePending()
    user = User(
        id=2,
        is_bot=False,
        first_name="user",
        last_name="user",
    )
    transaction_type = "premium_purchase"
    invoice_payload = "invoice_payload"
    paid_media = (
        PaidMediaVideo(
            video=Video(
                file_id="file_id",
                file_unique_id="file_unique_id",
                width=10,
                height=10,
                duration=10,
            )
        ),
    )
    paid_media_payload = "paid_media_payload"
    subscription_period = dtm.timedelta(days=1)
    gift = Gift(
        id="gift_id",
        sticker=Sticker(
            file_id="file_id",
            file_unique_id="file_unique_id",
            width=10,
            height=10,
            is_animated=False,
            is_video=False,
            type="type",
        ),
        total_count=42,
        remaining_count=42,
        star_count=42,
    )
    affiliate = AffiliateInfo(
        commission_per_mille=42,
        amount=42,
    )
    request_count = 42
    chat = Chat(
        id=3,
        type=Chat.CHANNEL,
    )
    premium_subscription_duration = 3


class TestTransactionPartnerWithoutRequest(TransactionPartnerTestBase):
    def test_slot_behaviour(self, transaction_partner):
        inst = transaction_partner
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_type_enum_conversion(self, transaction_partner):
        assert type(TransactionPartner("affiliate_program").type) is TransactionPartnerType
        assert TransactionPartner("unknown").type == "unknown"

    def test_de_json(self, offline_bot):
        data = {"type": "unknown"}
        transaction_partner = TransactionPartner.de_json(data, offline_bot)
        assert transaction_partner.api_kwargs == {}
        assert transaction_partner.type == "unknown"

    @pytest.mark.parametrize(
        ("tp_type", "subclass"),
        [
            ("affiliate_program", TransactionPartnerAffiliateProgram),
            ("fragment", TransactionPartnerFragment),
            ("user", TransactionPartnerUser),
            ("telegram_ads", TransactionPartnerTelegramAds),
            ("telegram_api", TransactionPartnerTelegramApi),
            ("other", TransactionPartnerOther),
            ("chat", TransactionPartnerChat),
        ],
    )
    def test_subclass(self, offline_bot, tp_type, subclass):
        json_dict = {
            "type": tp_type,
            "commission_per_mille": self.commission_per_mille,
            "user": self.user.to_dict(),
            "transaction_type": self.transaction_type,
            "request_count": self.request_count,
        }
        tp = TransactionPartner.de_json(json_dict, offline_bot)

        assert type(tp) is subclass
        assert set(tp.api_kwargs.keys()) == set(json_dict.keys()) - set(subclass.__slots__) - {
            "type"
        }
        assert tp.type == tp_type

    def test_to_dict(self, transaction_partner):
        data = transaction_partner.to_dict()
        assert data == {"type": self.type}

    def test_equality(self, transaction_partner):
        a = transaction_partner
        b = TransactionPartner(self.type)
        c = TransactionPartner("unknown")
        d = User(id=1, is_bot=False, first_name="user", last_name="user")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def transaction_partner_affiliate_program():
    return TransactionPartnerAffiliateProgram(
        commission_per_mille=TransactionPartnerTestBase.commission_per_mille,
        sponsor_user=TransactionPartnerTestBase.sponsor_user,
    )


class TestTransactionPartnerAffiliateProgramWithoutRequest(TransactionPartnerTestBase):
    type = TransactionPartnerType.AFFILIATE_PROGRAM

    def test_slot_behaviour(self, transaction_partner_affiliate_program):
        inst = transaction_partner_affiliate_program
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "commission_per_mille": self.commission_per_mille,
            "sponsor_user": self.sponsor_user.to_dict(),
        }
        tp = TransactionPartnerAffiliateProgram.de_json(json_dict, offline_bot)
        assert tp.api_kwargs == {}
        assert tp.type == "affiliate_program"
        assert tp.commission_per_mille == self.commission_per_mille
        assert tp.sponsor_user == self.sponsor_user

    def test_to_dict(self, transaction_partner_affiliate_program):
        json_dict = transaction_partner_affiliate_program.to_dict()
        assert json_dict["type"] == self.type
        assert json_dict["commission_per_mille"] == self.commission_per_mille
        assert json_dict["sponsor_user"] == self.sponsor_user.to_dict()

    def test_equality(self, transaction_partner_affiliate_program):
        a = transaction_partner_affiliate_program
        b = TransactionPartnerAffiliateProgram(
            commission_per_mille=self.commission_per_mille,
        )
        c = TransactionPartnerAffiliateProgram(
            commission_per_mille=0,
        )
        d = User(id=1, is_bot=False, first_name="user", last_name="user")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def transaction_partner_fragment():
    return TransactionPartnerFragment(
        withdrawal_state=TransactionPartnerTestBase.withdrawal_state,
    )


class TestTransactionPartnerFragmentWithoutRequest(TransactionPartnerTestBase):
    type = TransactionPartnerType.FRAGMENT

    def test_slot_behaviour(self, transaction_partner_fragment):
        inst = transaction_partner_fragment
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {"withdrawal_state": self.withdrawal_state.to_dict()}
        tp = TransactionPartnerFragment.de_json(json_dict, offline_bot)
        assert tp.api_kwargs == {}
        assert tp.type == "fragment"
        assert tp.withdrawal_state == self.withdrawal_state

    def test_to_dict(self, transaction_partner_fragment):
        json_dict = transaction_partner_fragment.to_dict()
        assert json_dict["type"] == self.type
        assert json_dict["withdrawal_state"] == self.withdrawal_state.to_dict()

    def test_equality(self, transaction_partner_fragment):
        a = transaction_partner_fragment
        b = TransactionPartnerFragment(withdrawal_state=self.withdrawal_state)
        c = TransactionPartnerFragment(withdrawal_state=None)
        d = User(id=1, is_bot=False, first_name="user", last_name="user")

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def transaction_partner_user():
    return TransactionPartnerUser(
        transaction_type=TransactionPartnerTestBase.transaction_type,
        user=TransactionPartnerTestBase.user,
        invoice_payload=TransactionPartnerTestBase.invoice_payload,
        paid_media=TransactionPartnerTestBase.paid_media,
        paid_media_payload=TransactionPartnerTestBase.paid_media_payload,
        subscription_period=TransactionPartnerTestBase.subscription_period,
        premium_subscription_duration=TransactionPartnerTestBase.premium_subscription_duration,
    )


class TestTransactionPartnerUserWithoutRequest(TransactionPartnerTestBase):
    type = TransactionPartnerType.USER

    def test_slot_behaviour(self, transaction_partner_user):
        inst = transaction_partner_user
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "user": self.user.to_dict(),
            "transaction_type": self.transaction_type,
            "invoice_payload": self.invoice_payload,
            "paid_media": [pm.to_dict() for pm in self.paid_media],
            "paid_media_payload": self.paid_media_payload,
            "subscription_period": self.subscription_period.total_seconds(),
            "premium_subscription_duration": self.premium_subscription_duration,
        }
        tp = TransactionPartnerUser.de_json(json_dict, offline_bot)
        assert tp.api_kwargs == {}
        assert tp.type == "user"
        assert tp.user == self.user
        assert tp.transaction_type == self.transaction_type
        assert tp.invoice_payload == self.invoice_payload
        assert tp.paid_media == self.paid_media
        assert tp.paid_media_payload == self.paid_media_payload
        assert tp.subscription_period == self.subscription_period
        assert tp.premium_subscription_duration == self.premium_subscription_duration

    def test_to_dict(self, transaction_partner_user):
        json_dict = transaction_partner_user.to_dict()
        assert json_dict["type"] == self.type
        assert json_dict["transaction_type"] == self.transaction_type
        assert json_dict["user"] == self.user.to_dict()
        assert json_dict["invoice_payload"] == self.invoice_payload
        assert json_dict["paid_media"] == [pm.to_dict() for pm in self.paid_media]
        assert json_dict["paid_media_payload"] == self.paid_media_payload
        assert json_dict["subscription_period"] == self.subscription_period.total_seconds()
        assert json_dict["premium_subscription_duration"] == self.premium_subscription_duration

    def test_equality(self, transaction_partner_user):
        a = transaction_partner_user
        b = TransactionPartnerUser(
            transaction_type=self.transaction_type,
            user=self.user,
        )
        c = TransactionPartnerUser(
            transaction_type=self.transaction_type,
            user=User(id=1, is_bot=False, first_name="user", last_name="user"),
        )
        d = User(id=1, is_bot=False, first_name="user", last_name="user")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def transaction_partner_other():
    return TransactionPartnerOther()


class TestTransactionPartnerOtherWithoutRequest(TransactionPartnerTestBase):
    type = TransactionPartnerType.OTHER

    def test_slot_behaviour(self, transaction_partner_other):
        inst = transaction_partner_other
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {}
        tp = TransactionPartnerOther.de_json(json_dict, offline_bot)
        assert tp.api_kwargs == {}
        assert tp.type == "other"

    def test_to_dict(self, transaction_partner_other):
        json_dict = transaction_partner_other.to_dict()
        assert json_dict == {"type": self.type}

    def test_equality(self, transaction_partner_other):
        a = transaction_partner_other
        b = TransactionPartnerOther()
        c = TransactionPartnerOther()
        d = User(id=1, is_bot=False, first_name="user", last_name="user")

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def transaction_partner_telegram_ads():
    return TransactionPartnerTelegramAds()


class TestTransactionPartnerTelegramAdsWithoutRequest(TransactionPartnerTestBase):
    type = TransactionPartnerType.TELEGRAM_ADS

    def test_slot_behaviour(self, transaction_partner_telegram_ads):
        inst = transaction_partner_telegram_ads
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {}
        tp = TransactionPartnerTelegramAds.de_json(json_dict, offline_bot)
        assert tp.api_kwargs == {}
        assert tp.type == "telegram_ads"

    def test_to_dict(self, transaction_partner_telegram_ads):
        json_dict = transaction_partner_telegram_ads.to_dict()
        assert json_dict == {"type": self.type}

    def test_equality(self, transaction_partner_telegram_ads):
        a = transaction_partner_telegram_ads
        b = TransactionPartnerTelegramAds()
        c = TransactionPartnerTelegramAds()
        d = User(id=1, is_bot=False, first_name="user", last_name="user")

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def transaction_partner_telegram_api():
    return TransactionPartnerTelegramApi(
        request_count=TransactionPartnerTestBase.request_count,
    )


class TestTransactionPartnerTelegramApiWithoutRequest(TransactionPartnerTestBase):
    type = TransactionPartnerType.TELEGRAM_API

    def test_slot_behaviour(self, transaction_partner_telegram_api):
        inst = transaction_partner_telegram_api
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {"request_count": self.request_count}
        tp = TransactionPartnerTelegramApi.de_json(json_dict, offline_bot)
        assert tp.api_kwargs == {}
        assert tp.type == "telegram_api"
        assert tp.request_count == self.request_count

    def test_to_dict(self, transaction_partner_telegram_api):
        json_dict = transaction_partner_telegram_api.to_dict()
        assert json_dict["type"] == self.type
        assert json_dict["request_count"] == self.request_count

    def test_equality(self, transaction_partner_telegram_api):
        a = transaction_partner_telegram_api
        b = TransactionPartnerTelegramApi(
            request_count=self.request_count,
        )
        c = TransactionPartnerTelegramApi(
            request_count=0,
        )
        d = User(id=1, is_bot=False, first_name="user", last_name="user")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def transaction_partner_chat():
    return TransactionPartnerChat(
        chat=TransactionPartnerTestBase.chat,
        gift=TransactionPartnerTestBase.gift,
    )


class TestTransactionPartnerChatWithoutRequest(TransactionPartnerTestBase):
    type = TransactionPartnerType.CHAT

    def test_slot_behaviour(self, transaction_partner_chat):
        inst = transaction_partner_chat
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "chat": self.chat.to_dict(),
            "gift": self.gift.to_dict(),
        }
        tp = TransactionPartnerChat.de_json(json_dict, offline_bot)
        assert tp.api_kwargs == {}
        assert tp.type == "chat"
        assert tp.chat == self.chat
        assert tp.gift == self.gift

    def test_to_dict(self, transaction_partner_chat):
        json_dict = transaction_partner_chat.to_dict()
        assert json_dict["type"] == self.type
        assert json_dict["chat"] == self.chat.to_dict()
        assert json_dict["gift"] == self.gift.to_dict()

    def test_equality(self, transaction_partner_chat):
        a = transaction_partner_chat
        b = TransactionPartnerChat(
            chat=self.chat,
            gift=self.gift,
        )
        c = TransactionPartnerChat(
            chat=Chat(id=1, type=Chat.CHANNEL),
        )
        d = Chat(id=1, type=Chat.CHANNEL)

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import (
    CallbackGame,
    CopyTextButton,
    InlineKeyboardButton,
    LoginUrl,
    SwitchInlineQueryChosenChat,
    WebAppInfo,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def inline_keyboard_button():
    return InlineKeyboardButton(
        InlineKeyboardButtonTestBase.text,
        url=InlineKeyboardButtonTestBase.url,
        callback_data=InlineKeyboardButtonTestBase.callback_data,
        switch_inline_query=InlineKeyboardButtonTestBase.switch_inline_query,
        switch_inline_query_current_chat=(
            InlineKeyboardButtonTestBase.switch_inline_query_current_chat
        ),
        callback_game=InlineKeyboardButtonTestBase.callback_game,
        pay=InlineKeyboardButtonTestBase.pay,
        login_url=InlineKeyboardButtonTestBase.login_url,
        web_app=InlineKeyboardButtonTestBase.web_app,
        switch_inline_query_chosen_chat=(
            InlineKeyboardButtonTestBase.switch_inline_query_chosen_chat
        ),
        copy_text=InlineKeyboardButtonTestBase.copy_text,
        style=InlineKeyboardButtonTestBase.style,
        icon_custom_emoji_id=InlineKeyboardButtonTestBase.icon_custom_emoji_id,
    )


class InlineKeyboardButtonTestBase:
    text = "text"
    url = "url"
    callback_data = "callback data"
    switch_inline_query = "switch_inline_query"
    switch_inline_query_current_chat = "switch_inline_query_current_chat"
    callback_game = CallbackGame()
    pay = True
    login_url = LoginUrl("http://google.com")
    web_app = WebAppInfo(url="https://example.com")
    switch_inline_query_chosen_chat = SwitchInlineQueryChosenChat("a_bot", True, False, True, True)
    copy_text = CopyTextButton("python-telegram-bot")
    style = "danger"
    icon_custom_emoji_id = "5237829955978547322"


class TestInlineKeyboardButtonWithoutRequest(InlineKeyboardButtonTestBase):
    def test_slot_behaviour(self, inline_keyboard_button):
        inst = inline_keyboard_button
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, inline_keyboard_button):
        assert inline_keyboard_button.text == self.text
        assert inline_keyboard_button.url == self.url
        assert inline_keyboard_button.callback_data == self.callback_data
        assert inline_keyboard_button.switch_inline_query == self.switch_inline_query
        assert (
            inline_keyboard_button.switch_inline_query_current_chat
            == self.switch_inline_query_current_chat
        )
        assert isinstance(inline_keyboard_button.callback_game, CallbackGame)
        assert inline_keyboard_button.pay == self.pay
        assert inline_keyboard_button.login_url == self.login_url
        assert inline_keyboard_button.web_app == self.web_app
        assert (
            inline_keyboard_button.switch_inline_query_chosen_chat
            == self.switch_inline_query_chosen_chat
        )
        assert inline_keyboard_button.copy_text == self.copy_text
        assert inline_keyboard_button.style == self.style
        assert inline_keyboard_button.icon_custom_emoji_id == self.icon_custom_emoji_id

    def test_to_dict(self, inline_keyboard_button):
        inline_keyboard_button_dict = inline_keyboard_button.to_dict()

        assert isinstance(inline_keyboard_button_dict, dict)
        assert inline_keyboard_button_dict["text"] == inline_keyboard_button.text
        assert inline_keyboard_button_dict["url"] == inline_keyboard_button.url
        assert inline_keyboard_button_dict["callback_data"] == inline_keyboard_button.callback_data
        assert (
            inline_keyboard_button_dict["switch_inline_query"]
            == inline_keyboard_button.switch_inline_query
        )
        assert (
            inline_keyboard_button_dict["switch_inline_query_current_chat"]
            == inline_keyboard_button.switch_inline_query_current_chat
        )
        assert (
            inline_keyboard_button_dict["callback_game"]
            == inline_keyboard_button.callback_game.to_dict()
        )
        assert inline_keyboard_button_dict["pay"] == inline_keyboard_button.pay
        assert (
            inline_keyboard_button_dict["login_url"] == inline_keyboard_button.login_url.to_dict()
        )
        assert inline_keyboard_button_dict["web_app"] == inline_keyboard_button.web_app.to_dict()
        assert (
            inline_keyboard_button_dict["switch_inline_query_chosen_chat"]
            == inline_keyboard_button.switch_inline_query_chosen_chat.to_dict()
        )
        assert (
            inline_keyboard_button_dict["copy_text"] == inline_keyboard_button.copy_text.to_dict()
        )
        assert inline_keyboard_button_dict["style"] == inline_keyboard_button.style
        assert (
            inline_keyboard_button_dict["icon_custom_emoji_id"]
            == inline_keyboard_button.icon_custom_emoji_id
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "text": self.text,
            "url": self.url,
            "callback_data": self.callback_data,
            "switch_inline_query": self.switch_inline_query,
            "switch_inline_query_current_chat": self.switch_inline_query_current_chat,
            "callback_game": self.callback_game.to_dict(),
            "web_app": self.web_app.to_dict(),
            "login_url": self.login_url.to_dict(),
            "pay": self.pay,
            "switch_inline_query_chosen_chat": self.switch_inline_query_chosen_chat.to_dict(),
            "copy_text": self.copy_text.to_dict(),
            "style": self.style,
            "icon_custom_emoji_id": self.icon_custom_emoji_id,
        }

        inline_keyboard_button = InlineKeyboardButton.de_json(json_dict, None)
        assert inline_keyboard_button.api_kwargs == {}
        assert inline_keyboard_button.text == self.text
        assert inline_keyboard_button.url == self.url
        assert inline_keyboard_button.callback_data == self.callback_data
        assert inline_keyboard_button.switch_inline_query == self.switch_inline_query
        assert (
            inline_keyboard_button.switch_inline_query_current_chat
            == self.switch_inline_query_current_chat
        )
        # CallbackGame has empty _id_attrs, so just test if the class is created.
        assert isinstance(inline_keyboard_button.callback_game, CallbackGame)
        assert inline_keyboard_button.pay == self.pay
        assert inline_keyboard_button.login_url == self.login_url
        assert inline_keyboard_button.web_app == self.web_app
        assert (
            inline_keyboard_button.switch_inline_query_chosen_chat
            == self.switch_inline_query_chosen_chat
        )
        assert inline_keyboard_button.copy_text == self.copy_text
        assert inline_keyboard_button.style == self.style
        assert inline_keyboard_button.icon_custom_emoji_id == self.icon_custom_emoji_id

    def test_equality(self):
        a = InlineKeyboardButton("text", callback_data="data")
        b = InlineKeyboardButton("text", callback_data="data")
        c = InlineKeyboardButton("texts", callback_data="data")
        d = InlineKeyboardButton("text", callback_data="info")
        e = InlineKeyboardButton("text", url="http://google.com")
        f = InlineKeyboardButton("text", web_app=WebAppInfo(url="https://ptb.org"))
        g = LoginUrl("http://google.com")
        h = InlineKeyboardButton("test", style="primary")
        i = InlineKeyboardButton("test", icon_custom_emoji_id="123")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert a != f
        assert hash(a) != hash(f)

        assert a != g
        assert hash(a) != hash(g)

        assert a != h
        assert hash(a) != hash(h)

        assert h != i
        assert hash(h) != hash(i)

    @pytest.mark.parametrize("callback_data", ["foo", 1, ("da", "ta"), object()])
    def test_update_callback_data(self, callback_data):
        button = InlineKeyboardButton(text="test", callback_data="data")
        button_b = InlineKeyboardButton(text="test", callback_data="data")

        assert button == button_b
        assert hash(button) == hash(button_b)

        button.update_callback_data(callback_data)
        assert button.callback_data is callback_data
        assert button != button_b
        assert hash(button) != hash(button_b)

        button_b.update_callback_data(callback_data)
        assert button_b.callback_data is callback_data
        assert button == button_b
        assert hash(button) == hash(button_b)

        button.update_callback_data({})
        assert button.callback_data == {}
        with pytest.raises(TypeError, match="unhashable"):
            hash(button)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram ChatMemberUpdated."""

import datetime as dtm
from typing import TYPE_CHECKING

from telegram._chat import Chat
from telegram._chatinvitelink import ChatInviteLink
from telegram._chatmember import ChatMember
from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.datetime import extract_tzinfo_from_defaults, from_timestamp
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class ChatMemberUpdated(TelegramObject):
    """This object represents changes in the status of a chat member.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`chat`, :attr:`from_user`, :attr:`date`,
    :attr:`old_chat_member` and :attr:`new_chat_member` are equal.

    .. versionadded:: 13.4

    Note:
        In Python :keyword:`from` is a reserved word. Use :paramref:`from_user` instead.

    Examples:
        :any:`Chat Member Bot <examples.chatmemberbot>`

    Args:
        chat (:class:`telegram.Chat`): Chat the user belongs to.
        from_user (:class:`telegram.User`): Performer of the action, which resulted in the change.
        date (:class:`datetime.datetime`): Date the change was done in Unix time. Converted to
            :class:`datetime.datetime`.

            .. versionchanged:: 20.3
                |datetime_localization|
        old_chat_member (:class:`telegram.ChatMember`): Previous information about the chat member.
        new_chat_member (:class:`telegram.ChatMember`): New information about the chat member.
        invite_link (:class:`telegram.ChatInviteLink`, optional): Chat invite link, which was used
            by the user to join the chat. For joining by invite link events only.
        via_chat_folder_invite_link (:obj:`bool`, optional): :obj:`True`, if the user joined the
            chat via a chat folder invite link

            .. versionadded:: 20.3
        via_join_request (:obj:`bool`, optional): :obj:`True`, if the user joined the chat after
            sending a direct join request without using an invite link and being approved by
            an administrator

            .. versionadded:: 21.2

    Attributes:
        chat (:class:`telegram.Chat`): Chat the user belongs to.
        from_user (:class:`telegram.User`): Performer of the action, which resulted in the change.
        date (:class:`datetime.datetime`): Date the change was done in Unix time. Converted to
            :class:`datetime.datetime`.

            .. versionchanged:: 20.3
                |datetime_localization|
        old_chat_member (:class:`telegram.ChatMember`): Previous information about the chat member.
        new_chat_member (:class:`telegram.ChatMember`): New information about the chat member.
        invite_link (:class:`telegram.ChatInviteLink`): Optional. Chat invite link, which was used
            by the user to join the chat. For joining by invite link events only.
        via_chat_folder_invite_link (:obj:`bool`): Optional. :obj:`True`, if the user joined the
            chat via a chat folder invite link

            .. versionadded:: 20.3
        via_join_request (:obj:`bool`): Optional. :obj:`True`, if the user joined the chat after
            sending a direct join request without using an invite link and being approved
            by an administrator

            .. versionadded:: 21.2

    """

    __slots__ = (
        "chat",
        "date",
        "from_user",
        "invite_link",
        "new_chat_member",
        "old_chat_member",
        "via_chat_folder_invite_link",
        "via_join_request",
    )

    def __init__(
        self,
        chat: Chat,
        from_user: User,
        date: dtm.datetime,
        old_chat_member: ChatMember,
        new_chat_member: ChatMember,
        invite_link: ChatInviteLink | None = None,
        via_chat_folder_invite_link: bool | None = None,
        via_join_request: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.chat: Chat = chat
        self.from_user: User = from_user
        self.date: dtm.datetime = date
        self.old_chat_member: ChatMember = old_chat_member
        self.new_chat_member: ChatMember = new_chat_member
        self.via_chat_folder_invite_link: bool | None = via_chat_folder_invite_link

        # Optionals
        self.invite_link: ChatInviteLink | None = invite_link
        self.via_join_request: bool | None = via_join_request

        self._id_attrs = (
            self.chat,
            self.from_user,
            self.date,
            self.old_chat_member,
            self.new_chat_member,
        )

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "ChatMemberUpdated":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)

        data["chat"] = de_json_optional(data.get("chat"), Chat, bot)
        data["from_user"] = de_json_optional(data.pop("from", None), User, bot)
        data["date"] = from_timestamp(data.get("date"), tzinfo=loc_tzinfo)
        data["old_chat_member"] = de_json_optional(data.get("old_chat_member"), ChatMember, bot)
        data["new_chat_member"] = de_json_optional(data.get("new_chat_member"), ChatMember, bot)
        data["invite_link"] = de_json_optional(data.get("invite_link"), ChatInviteLink, bot)

        return super().de_json(data=data, bot=bot)

    def _get_attribute_difference(self, attribute: str) -> tuple[object, object]:
        try:
            old = self.old_chat_member[attribute]
        except KeyError:
            old = None

        try:
            new = self.new_chat_member[attribute]
        except KeyError:
            new = None

        return old, new

    def difference(
        self,
    ) -> dict[
        str,
        tuple[str | bool | dtm.datetime | User, str | bool | dtm.datetime | User],
    ]:
        """Computes the difference between :attr:`old_chat_member` and :attr:`new_chat_member`.

        Example:
            .. code:: pycon

                >>> chat_member_updated.difference()
                {'custom_title': ('old title', 'new title')}

        Note:
            To determine, if the :attr:`telegram.ChatMember.user` attribute has changed, *every*
            attribute of the user will be checked.

        .. versionadded:: 13.5

        Returns:
            dict[:obj:`str`, tuple[:class:`object`, :class:`object`]]: A dictionary mapping
            attribute names to tuples of the form ``(old_value, new_value)``
        """
        # we first get the names of the attributes that have changed
        # user.to_dict() is unhashable, so that needs some special casing further down
        old_dict = self.old_chat_member.to_dict()
        old_user_dict = old_dict.pop("user")
        new_dict = self.new_chat_member.to_dict()
        new_user_dict = new_dict.pop("user")

        # Generator for speed: we only need to iterate over it once
        # we can't directly use the values from old_dict ^ new_dict b/c that set is unordered
        attributes = (entry[0] for entry in set(old_dict.items()) ^ set(new_dict.items()))

        result = {attribute: self._get_attribute_difference(attribute) for attribute in attributes}
        if old_user_dict != new_user_dict:
            result["user"] = (self.old_chat_member.user, self.new_chat_member.user)

        return result  # type: ignore[return-value]



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import Bot, InlineQuery, Location, Update, User
from tests.auxil.bot_method_checks import (
    check_defaults_handling,
    check_shortcut_call,
    check_shortcut_signature,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def inline_query(bot):
    ilq = InlineQuery(
        InlineQueryTestBase.id_,
        InlineQueryTestBase.from_user,
        InlineQueryTestBase.query,
        InlineQueryTestBase.offset,
        location=InlineQueryTestBase.location,
    )
    ilq.set_bot(bot)
    return ilq


class InlineQueryTestBase:
    id_ = 1234
    from_user = User(1, "First name", False)
    query = "query text"
    offset = "offset"
    location = Location(8.8, 53.1)


class TestInlineQueryWithoutRequest(InlineQueryTestBase):
    def test_slot_behaviour(self, inline_query):
        for attr in inline_query.__slots__:
            assert getattr(inline_query, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inline_query)) == len(set(mro_slots(inline_query))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "id": self.id_,
            "from": self.from_user.to_dict(),
            "query": self.query,
            "offset": self.offset,
            "location": self.location.to_dict(),
        }
        inline_query_json = InlineQuery.de_json(json_dict, offline_bot)
        assert inline_query_json.api_kwargs == {}

        assert inline_query_json.id == self.id_
        assert inline_query_json.from_user == self.from_user
        assert inline_query_json.location == self.location
        assert inline_query_json.query == self.query
        assert inline_query_json.offset == self.offset

    def test_to_dict(self, inline_query):
        inline_query_dict = inline_query.to_dict()

        assert isinstance(inline_query_dict, dict)
        assert inline_query_dict["id"] == inline_query.id
        assert inline_query_dict["from"] == inline_query.from_user.to_dict()
        assert inline_query_dict["location"] == inline_query.location.to_dict()
        assert inline_query_dict["query"] == inline_query.query
        assert inline_query_dict["offset"] == inline_query.offset

    def test_equality(self):
        a = InlineQuery(self.id_, User(1, "", False), "", "")
        b = InlineQuery(self.id_, User(1, "", False), "", "")
        c = InlineQuery(self.id_, User(0, "", False), "", "")
        d = InlineQuery(0, User(1, "", False), "", "")
        e = Update(self.id_)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_answer_error(self, inline_query):
        with pytest.raises(ValueError, match="mutually exclusive"):
            await inline_query.answer(results=[], auto_pagination=True, current_offset="foobar")

    async def test_answer(self, monkeypatch, inline_query):
        async def make_assertion(*_, **kwargs):
            return kwargs["inline_query_id"] == inline_query.id

        assert check_shortcut_signature(
            InlineQuery.answer, Bot.answer_inline_query, ["inline_query_id"], ["auto_pagination"]
        )
        assert await check_shortcut_call(
            inline_query.answer, inline_query.get_bot(), "answer_inline_query"
        )
        assert await check_defaults_handling(inline_query.answer, inline_query.get_bot())

        monkeypatch.setattr(inline_query.get_bot(), "answer_inline_query", make_assertion)
        assert await inline_query.answer(results=[])

    async def test_answer_auto_pagination(self, monkeypatch, inline_query):
        async def make_assertion(*_, **kwargs):
            inline_query_id_matches = kwargs["inline_query_id"] == inline_query.id
            offset_matches = kwargs.get("current_offset") == inline_query.offset
            return offset_matches and inline_query_id_matches

        monkeypatch.setattr(inline_query.get_bot(), "answer_inline_query", make_assertion)
        assert await inline_query.answer(results=[], auto_pagination=True)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import json

import pytest

from telegram.ext import DictPersistence
from tests.auxil.slots import mro_slots


@pytest.fixture(autouse=True)
def _reset_callback_data_cache(cdc_bot):
    yield
    cdc_bot.callback_data_cache.clear_callback_data()
    cdc_bot.callback_data_cache.clear_callback_queries()


@pytest.fixture
def bot_data():
    return {"test1": "test2", "test3": {"test4": "test5"}}


@pytest.fixture
def chat_data():
    return {-12345: {"test1": "test2", "test3": {"test4": "test5"}}, -67890: {3: "test4"}}


@pytest.fixture
def user_data():
    return {12345: {"test1": "test2", "test3": {"test4": "test5"}}, 67890: {3: "test4"}}


@pytest.fixture
def callback_data():
    return [("test1", 1000, {"button1": "test0", "button2": "test1"})], {"test1": "test2"}


@pytest.fixture
def conversations():
    return {
        "name1": {(123, 123): 3, (456, 654): 4},
        "name2": {(123, 321): 1, (890, 890): 2},
        "name3": {(123, 321): 1, (890, 890): 2},
    }


@pytest.fixture
def user_data_json(user_data):
    return json.dumps(user_data)


@pytest.fixture
def chat_data_json(chat_data):
    return json.dumps(chat_data)


@pytest.fixture
def bot_data_json(bot_data):
    return json.dumps(bot_data)


@pytest.fixture
def callback_data_json(callback_data):
    return json.dumps(callback_data)


@pytest.fixture
def conversations_json(conversations):
    return """{"name1": {"[123, 123]": 3, "[456, 654]": 4}, "name2":
              {"[123, 321]": 1, "[890, 890]": 2}, "name3":
              {"[123, 321]": 1, "[890, 890]": 2}}"""


class TestDictPersistence:
    """Just tests the DictPersistence interface. Integration of persistence into Applictation
    is tested in TestBasePersistence!"""

    async def test_slot_behaviour(self):
        inst = DictPersistence()
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    async def test_no_json_given(self):
        dict_persistence = DictPersistence()
        assert await dict_persistence.get_user_data() == {}
        assert await dict_persistence.get_chat_data() == {}
        assert await dict_persistence.get_bot_data() == {}
        assert await dict_persistence.get_callback_data() is None
        assert await dict_persistence.get_conversations("noname") == {}

    async def test_bad_json_string_given(self):
        bad_user_data = "thisisnojson99900()))("
        bad_chat_data = "thisisnojson99900()))("
        bad_bot_data = "thisisnojson99900()))("
        bad_callback_data = "thisisnojson99900()))("
        bad_conversations = "thisisnojson99900()))("
        with pytest.raises(TypeError, match="user_data"):
            DictPersistence(user_data_json=bad_user_data)
        with pytest.raises(TypeError, match="chat_data"):
            DictPersistence(chat_data_json=bad_chat_data)
        with pytest.raises(TypeError, match="bot_data"):
            DictPersistence(bot_data_json=bad_bot_data)
        with pytest.raises(TypeError, match="callback_data"):
            DictPersistence(callback_data_json=bad_callback_data)
        with pytest.raises(TypeError, match="conversations"):
            DictPersistence(conversations_json=bad_conversations)

    async def test_invalid_json_string_given(self):
        bad_user_data = '["this", "is", "json"]'
        bad_chat_data = '["this", "is", "json"]'
        bad_bot_data = '["this", "is", "json"]'
        bad_conversations = '["this", "is", "json"]'
        bad_callback_data_1 = '[[["str", 3.14, {"di": "ct"}]], "is"]'
        bad_callback_data_2 = '[[["str", "non-float", {"di": "ct"}]], {"di": "ct"}]'
        bad_callback_data_3 = '[[[{"not": "a str"}, 3.14, {"di": "ct"}]], {"di": "ct"}]'
        bad_callback_data_4 = '[[["wrong", "length"]], {"di": "ct"}]'
        bad_callback_data_5 = '["this", "is", "json"]'
        with pytest.raises(TypeError, match="user_data"):
            DictPersistence(user_data_json=bad_user_data)
        with pytest.raises(TypeError, match="chat_data"):
            DictPersistence(chat_data_json=bad_chat_data)
        with pytest.raises(TypeError, match="bot_data"):
            DictPersistence(bot_data_json=bad_bot_data)
        for bad_callback_data in [
            bad_callback_data_1,
            bad_callback_data_2,
            bad_callback_data_3,
            bad_callback_data_4,
            bad_callback_data_5,
        ]:
            with pytest.raises(TypeError, match="callback_data"):
                DictPersistence(callback_data_json=bad_callback_data)
        with pytest.raises(TypeError, match="conversations"):
            DictPersistence(conversations_json=bad_conversations)

    async def test_good_json_input(
        self, user_data_json, chat_data_json, bot_data_json, conversations_json, callback_data_json
    ):
        dict_persistence = DictPersistence(
            user_data_json=user_data_json,
            chat_data_json=chat_data_json,
            bot_data_json=bot_data_json,
            conversations_json=conversations_json,
            callback_data_json=callback_data_json,
        )
        user_data = await dict_persistence.get_user_data()
        assert isinstance(user_data, dict)
        assert user_data[12345]["test1"] == "test2"
        assert user_data[67890][3] == "test4"

        chat_data = await dict_persistence.get_chat_data()
        assert isinstance(chat_data, dict)
        assert chat_data[-12345]["test1"] == "test2"
        assert chat_data[-67890][3] == "test4"

        bot_data = await dict_persistence.get_bot_data()
        assert isinstance(bot_data, dict)
        assert bot_data["test1"] == "test2"
        assert bot_data["test3"]["test4"] == "test5"
        assert "test6" not in bot_data

        callback_data = await dict_persistence.get_callback_data()

        assert isinstance(callback_data, tuple)
        assert callback_data[0] == [("test1", 1000, {"button1": "test0", "button2": "test1"})]
        assert callback_data[1] == {"test1": "test2"}

        conversation1 = await dict_persistence.get_conversations("name1")
        assert isinstance(conversation1, dict)
        assert conversation1[(123, 123)] == 3
        assert conversation1[(456, 654)] == 4
        with pytest.raises(KeyError):
            conversation1[(890, 890)]
        conversation2 = await dict_persistence.get_conversations("name2")
        assert isinstance(conversation1, dict)
        assert conversation2[(123, 321)] == 1
        assert conversation2[(890, 890)] == 2
        with pytest.raises(KeyError):
            conversation2[(123, 123)]

    async def test_good_json_input_callback_data_none(self):
        dict_persistence = DictPersistence(callback_data_json="null")
        assert dict_persistence.callback_data is None
        assert dict_persistence.callback_data_json == "null"

    async def test_dict_outputs(
        self,
        user_data,
        user_data_json,
        chat_data,
        chat_data_json,
        bot_data,
        bot_data_json,
        callback_data_json,
        conversations,
        conversations_json,
    ):
        dict_persistence = DictPersistence(
            user_data_json=user_data_json,
            chat_data_json=chat_data_json,
            bot_data_json=bot_data_json,
            callback_data_json=callback_data_json,
            conversations_json=conversations_json,
        )
        assert dict_persistence.user_data == user_data
        assert dict_persistence.chat_data == chat_data
        assert dict_persistence.bot_data == bot_data
        assert dict_persistence.bot_data == bot_data
        assert dict_persistence.conversations == conversations

    async def test_json_outputs(
        self, user_data_json, chat_data_json, bot_data_json, callback_data_json, conversations_json
    ):
        dict_persistence = DictPersistence(
            user_data_json=user_data_json,
            chat_data_json=chat_data_json,
            bot_data_json=bot_data_json,
            callback_data_json=callback_data_json,
            conversations_json=conversations_json,
        )
        assert dict_persistence.user_data_json == user_data_json
        assert dict_persistence.chat_data_json == chat_data_json
        assert dict_persistence.callback_data_json == callback_data_json
        assert dict_persistence.conversations_json == conversations_json

    async def test_updating(
        self,
        user_data_json,
        chat_data_json,
        bot_data_json,
        callback_data,
        callback_data_json,
        conversations,
        conversations_json,
    ):
        dict_persistence = DictPersistence(
            user_data_json=user_data_json,
            chat_data_json=chat_data_json,
            bot_data_json=bot_data_json,
            callback_data_json=callback_data_json,
            conversations_json=conversations_json,
        )

        user_data = await dict_persistence.get_user_data()
        user_data[12345]["test3"]["test4"] = "test6"
        assert dict_persistence.user_data != user_data
        assert dict_persistence.user_data_json != json.dumps(user_data)
        await dict_persistence.update_user_data(12345, user_data[12345])
        assert dict_persistence.user_data == user_data
        assert dict_persistence.user_data_json == json.dumps(user_data)
        await dict_persistence.drop_user_data(67890)
        assert 67890 not in dict_persistence.user_data
        dict_persistence._user_data = None
        await dict_persistence.drop_user_data(123)
        assert isinstance(await dict_persistence.get_user_data(), dict)

        chat_data = await dict_persistence.get_chat_data()
        chat_data[-12345]["test3"]["test4"] = "test6"
        assert dict_persistence.chat_data != chat_data
        assert dict_persistence.chat_data_json != json.dumps(chat_data)
        await dict_persistence.update_chat_data(-12345, chat_data[-12345])
        assert dict_persistence.chat_data == chat_data
        assert dict_persistence.chat_data_json == json.dumps(chat_data)
        await dict_persistence.drop_chat_data(-67890)
        assert -67890 not in dict_persistence.chat_data
        dict_persistence._chat_data = None
        await dict_persistence.drop_chat_data(123)
        assert isinstance(await dict_persistence.get_chat_data(), dict)

        bot_data = await dict_persistence.get_bot_data()
        bot_data["test3"]["test4"] = "test6"
        assert dict_persistence.bot_data != bot_data
        assert dict_persistence.bot_data_json != json.dumps(bot_data)
        await dict_persistence.update_bot_data(bot_data)
        assert dict_persistence.bot_data == bot_data
        assert dict_persistence.bot_data_json == json.dumps(bot_data)

        callback_data = await dict_persistence.get_callback_data()
        callback_data[1]["test3"] = "test4"
        callback_data[0][0][2]["button2"] = "test41"
        assert dict_persistence.callback_data != callback_data
        assert dict_persistence.callback_data_json != json.dumps(callback_data)
        await dict_persistence.update_callback_data(callback_data)
        assert dict_persistence.callback_data == callback_data
        assert dict_persistence.callback_data_json == json.dumps(callback_data)

        conversation1 = await dict_persistence.get_conversations("name1")
        conversation1[(123, 123)] = 5
        assert dict_persistence.conversations["name1"] != conversation1
        await dict_persistence.update_conversation("name1", (123, 123), 5)
        assert dict_persistence.conversations["name1"] == conversation1
        conversations["name1"][(123, 123)] = 5
        assert (
            dict_persistence.conversations_json
            == DictPersistence._encode_conversations_to_json(conversations)
        )
        assert await dict_persistence.get_conversations("name1") == conversation1

        dict_persistence._conversations = None
        await dict_persistence.update_conversation("name1", (123, 123), 5)
        assert dict_persistence.conversations["name1"] == {(123, 123): 5}
        assert await dict_persistence.get_conversations("name1") == {(123, 123): 5}
        assert (
            dict_persistence.conversations_json
            == DictPersistence._encode_conversations_to_json({"name1": {(123, 123): 5}})
        )

    async def test_no_data_on_init(
        self, bot_data, user_data, chat_data, conversations, callback_data
    ):
        dict_persistence = DictPersistence()

        assert dict_persistence.user_data is None
        assert dict_persistence.chat_data is None
        assert dict_persistence.bot_data is None
        assert dict_persistence.conversations is None
        assert dict_persistence.callback_data is None
        assert dict_persistence.user_data_json == "null"
        assert dict_persistence.chat_data_json == "null"
        assert dict_persistence.bot_data_json == "null"
        assert dict_persistence.conversations_json == "null"
        assert dict_persistence.callback_data_json == "null"

        await dict_persistence.update_bot_data(bot_data)
        await dict_persistence.update_user_data(12345, user_data[12345])
        await dict_persistence.update_chat_data(-12345, chat_data[-12345])
        await dict_persistence.update_conversation("name", (1, 1), "new_state")
        await dict_persistence.update_callback_data(callback_data)

        assert dict_persistence.user_data[12345] == user_data[12345]
        assert dict_persistence.chat_data[-12345] == chat_data[-12345]
        assert dict_persistence.bot_data == bot_data
        assert dict_persistence.conversations["name"] == {(1, 1): "new_state"}
        assert dict_persistence.callback_data == callback_data

    async def test_no_json_dumping_if_data_did_not_change(
        self, bot_data, user_data, chat_data, conversations, callback_data, monkeypatch
    ):
        dict_persistence = DictPersistence()

        await dict_persistence.update_bot_data(bot_data)
        await dict_persistence.update_user_data(12345, user_data[12345])
        await dict_persistence.update_chat_data(-12345, chat_data[-12345])
        await dict_persistence.update_conversation("name", (1, 1), "new_state")
        await dict_persistence.update_callback_data(callback_data)

        assert dict_persistence.user_data_json == json.dumps({12345: user_data[12345]})
        assert dict_persistence.chat_data_json == json.dumps({-12345: chat_data[-12345]})
        assert dict_persistence.bot_data_json == json.dumps(bot_data)
        assert (
            dict_persistence.conversations_json
            == DictPersistence._encode_conversations_to_json({"name": {(1, 1): "new_state"}})
        )
        assert dict_persistence.callback_data_json == json.dumps(callback_data)

        flag = False

        def dumps(*args, **kwargs):
            nonlocal flag
            flag = True

        # Since the data doesn't change, json.dumps shoduln't be called beyond this point!
        monkeypatch.setattr(json, "dumps", dumps)

        await dict_persistence.update_bot_data(bot_data)
        await dict_persistence.update_user_data(12345, user_data[12345])
        await dict_persistence.update_chat_data(-12345, chat_data[-12345])
        await dict_persistence.update_conversation("name", (1, 1), "new_state")
        await dict_persistence.update_callback_data(callback_data)

        assert not flag



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram File."""

import shutil
import urllib.parse as urllib_parse
from base64 import b64decode
from pathlib import Path
from typing import TYPE_CHECKING, BinaryIO

from telegram._passport.credentials import decrypt
from telegram._telegramobject import TelegramObject
from telegram._utils.defaultvalue import DEFAULT_NONE
from telegram._utils.files import is_local_file
from telegram._utils.types import FilePathInput, JSONDict, ODVInput

if TYPE_CHECKING:
    from telegram import FileCredentials


class File(TelegramObject):
    """
    This object represents a file ready to be downloaded. The file can be e.g. downloaded with
    :attr:`download_to_drive`. It is guaranteed that the link will be valid for at least 1 hour.
    When the link expires, a new one can be requested by calling :meth:`telegram.Bot.get_file`.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`file_unique_id` is equal.

    .. versionchanged:: 20.0
        ``download`` was split into :meth:`download_to_drive` and :meth:`download_to_memory`.

    Note:
        * Maximum file size to download is
          :tg-const:`telegram.constants.FileSizeLimit.FILESIZE_DOWNLOAD`.
        * If you obtain an instance of this class from :attr:`telegram.PassportFile.get_file`,
          then it will automatically be decrypted as it downloads when you call e.g.
          :meth:`download_to_drive`.

    Args:
        file_id (:obj:`str`): Identifier for this file, which can be used to download
            or reuse the file.
        file_unique_id (:obj:`str`): Unique identifier for this file, which
            is supposed to be the same over time and for different bots.
            Can't be used to download or reuse the file.
        file_size (:obj:`int`, optional): File size in bytes, if known.
        file_path (:obj:`str`, optional): File path. Use e.g. :meth:`download_to_drive` to get the
            file.

    Attributes:
        file_id (:obj:`str`): Identifier for this file, which can be used to download
            or reuse the file.
        file_unique_id (:obj:`str`): Unique identifier for this file, which
            is supposed to be the same over time and for different bots.
            Can't be used to download or reuse the file.
        file_size (:obj:`int`): Optional. File size in bytes, if known.
        file_path (:obj:`str`): Optional. File path. Use e.g. :meth:`download_to_drive` to get the
            file.
    """

    __slots__ = (
        "_credentials",
        "file_id",
        "file_path",
        "file_size",
        "file_unique_id",
    )

    def __init__(
        self,
        file_id: str,
        file_unique_id: str,
        file_size: int | None = None,
        file_path: str | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        # Required
        self.file_id: str = str(file_id)
        self.file_unique_id: str = str(file_unique_id)
        # Optionals
        self.file_size: int | None = file_size
        self.file_path: str | None = file_path

        self._credentials: FileCredentials | None = None

        self._id_attrs = (self.file_unique_id,)

        self._freeze()

    def _get_encoded_url(self) -> str:
        """Convert any UTF-8 char in :obj:`File.file_path` into a url encoded ASCII string."""
        sres = urllib_parse.urlsplit(str(self.file_path))
        return urllib_parse.urlunsplit(
            urllib_parse.SplitResult(
                sres.scheme, sres.netloc, urllib_parse.quote(sres.path), sres.query, sres.fragment
            )
        )

    def _prepare_decrypt(self, buf: bytes) -> bytes:
        return decrypt(b64decode(self._credentials.secret), b64decode(self._credentials.hash), buf)

    async def download_to_drive(
        self,
        custom_path: FilePathInput | None = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
    ) -> Path:
        """
        Download this file. By default, the file is saved in the current working directory with
        :attr:`file_path` as file name. If :paramref:`custom_path` is supplied as a :obj:`str` or
        :obj:`pathlib.Path`, it will be saved to that path.

        Note:
            If :paramref:`custom_path` isn't provided and :attr:`file_path` is the path of a
            local file (which is the case when a Bot API Server is running in local mode), this
            method will just return the path.

            The only exception to this are encrypted files (e.g. a passport file). For these, a
            file with the prefix `decrypted_` will be created in the same directory as the
            original file in order to decrypt the file without changing the existing one
            in-place.

        .. seealso:: :wiki:`Working with Files and Media <Working-with-Files-and-Media>`

        .. versionchanged:: 20.0

            * :paramref:`custom_path` parameter now also accepts :class:`pathlib.Path` as argument.
            * Returns :class:`pathlib.Path` object in cases where previously a :obj:`str` was
              returned.
            * This method was previously called ``download``. It was split into
              :meth:`download_to_drive` and :meth:`download_to_memory`.

        .. versionchanged:: 21.7
            Raises :exc:`RuntimeError` if :attr:`file_path` is not set. Note that files without
            a :attr:`file_path` could never be downloaded, as this attribute is mandatory for that
            operation.

        Args:
            custom_path (:class:`pathlib.Path` | :obj:`str` , optional): The path where the file
                will be saved to. If not specified, will be saved in the current working directory
                with :attr:`file_path` as file name or the :attr:`file_id` if :attr:`file_path`
                is not set.

        Keyword Args:
            read_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.read_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.
            write_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.write_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.
            connect_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.connect_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.
            pool_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.pool_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.

        Returns:
            :class:`pathlib.Path`: Returns the Path object the file was downloaded to.

        Raises:
            RuntimeError: If :attr:`file_path` is not set.

        """
        if not self.file_path:
            raise RuntimeError("No `file_path` available for this file. Can not download.")

        local_file = is_local_file(self.file_path)
        url = None if local_file else self._get_encoded_url()

        # if _credentials exists we want to decrypt the file
        if local_file and self._credentials:
            file_to_decrypt = Path(self.file_path)
            buf = self._prepare_decrypt(file_to_decrypt.read_bytes())
            if custom_path is not None:
                path = Path(custom_path)
            else:
                path = Path(str(file_to_decrypt.parent) + "/decrypted_" + file_to_decrypt.name)
            path.write_bytes(buf)
            return path

        if custom_path is not None and local_file:
            shutil.copyfile(self.file_path, str(custom_path))
            return Path(custom_path)

        if custom_path:
            filename = Path(custom_path)
        elif local_file:
            return Path(self.file_path)
        else:
            filename = Path(Path(self.file_path).name)

        buf = await self.get_bot().request.retrieve(
            url,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
        )
        if self._credentials:
            buf = self._prepare_decrypt(buf)
        filename.write_bytes(buf)
        return filename

    async def download_to_memory(
        self,
        out: BinaryIO,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
    ) -> None:
        """
        Download this file into memory. :paramref:`out` needs to be supplied with a
        :obj:`io.BufferedIOBase`, the file contents will be saved to that object using the
        :obj:`out.write<io.BufferedIOBase.write>` method.

        .. seealso:: :wiki:`Working with Files and Media <Working-with-Files-and-Media>`

        Hint:
            If you want to immediately read the data from ``out`` after calling this method, you
            should call ``out.seek(0)`` first. See also :meth:`io.IOBase.seek`.

        .. versionadded:: 20.0

        .. versionchanged:: 21.7
            Raises :exc:`RuntimeError` if :attr:`file_path` is not set. Note that files without
            a :attr:`file_path` could never be downloaded, as this attribute is mandatory for that
            operation.

        Args:
            out (:obj:`io.BufferedIOBase`): A file-like object. Must be opened for writing in
                binary mode.

        Keyword Args:
            read_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.read_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.
            write_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.write_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.
            connect_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.connect_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.
            pool_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.pool_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.

        Raises:
            RuntimeError: If :attr:`file_path` is not set.
        """
        if not self.file_path:
            raise RuntimeError("No `file_path` available for this file. Can not download.")

        local_file = is_local_file(self.file_path)
        url = None if local_file else self._get_encoded_url()
        path = Path(self.file_path) if local_file else None
        if local_file:
            buf = path.read_bytes()
        else:
            buf = await self.get_bot().request.retrieve(
                url,
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
            )
        if self._credentials:
            buf = self._prepare_decrypt(buf)
        out.write(buf)

    async def download_as_bytearray(
        self,
        buf: bytearray | None = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
    ) -> bytearray:
        """Download this file and return it as a bytearray.

        .. versionchanged:: 21.7
            Raises :exc:`RuntimeError` if :attr:`file_path` is not set. Note that files without
            a :attr:`file_path` could never be downloaded, as this attribute is mandatory for that
            operation.

        Args:
            buf (:obj:`bytearray`, optional): Extend the given bytearray with the downloaded data.

        Keyword Args:
            read_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.read_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.

                .. versionadded:: 20.0
            write_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.write_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.

                .. versionadded:: 20.0
            connect_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.connect_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.

                .. versionadded:: 20.0
            pool_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to
                :paramref:`telegram.request.BaseRequest.post.pool_timeout`. Defaults to
                :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.

                .. versionadded:: 20.0

        Returns:
            :obj:`bytearray`: The same object as :paramref:`buf` if it was specified. Otherwise a
            newly allocated :obj:`bytearray`.

        Raises:
            RuntimeError: If :attr:`file_path` is not set.

        """
        if not self.file_path:
            raise RuntimeError("No `file_path` available for this file. Can not download.")

        if buf is None:
            buf = bytearray()

        if is_local_file(self.file_path):
            bytes_data = Path(self.file_path).read_bytes()
        else:
            bytes_data = await self.get_bot().request.retrieve(
                self._get_encoded_url(),
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
            )
        if self._credentials:
            buf.extend(self._prepare_decrypt(bytes_data))
        else:
            buf.extend(bytes_data)
        return buf

    def set_credentials(self, credentials: "FileCredentials") -> None:
        """Sets the passport credentials for the file.

        Args:
            credentials (:class:`telegram.FileCredentials`): The credentials.
        """
        self._credentials = credentials



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program. If not, see [http://www.gnu.org/licenses/].
# pylint: disable=redefined-builtin
"""This module contains the classes for Telegram Stars transactions."""

import datetime as dtm
from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._telegramobject import TelegramObject
from telegram._utils.argumentparsing import de_json_optional, de_list_optional, parse_sequence_arg
from telegram._utils.datetime import extract_tzinfo_from_defaults, from_timestamp
from telegram._utils.types import JSONDict

from .transactionpartner import TransactionPartner

if TYPE_CHECKING:
    from telegram import Bot


class StarTransaction(TelegramObject):
    """Describes a Telegram Star transaction.
    Note that if the buyer initiates a chargeback with the payment provider from whom they
    acquired Stars (e.g., Apple, Google) following this transaction, the refunded Stars will be
    deducted from the bot's balance. This is outside of Telegram's control.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`id`, :attr:`source`, and :attr:`receiver` are equal.

    .. versionadded:: 21.4

    Args:
        id (:obj:`str`): Unique identifier of the transaction. Coincides with the identifer
            of the original transaction for refund transactions.
            Coincides with :attr:`SuccessfulPayment.telegram_payment_charge_id` for
            successful incoming payments from users.
        amount (:obj:`int`): Integer amount of Telegram Stars transferred by the transaction.
        nanostar_amount (:obj:`int`, optional): The number of
            :tg-const:`~telegram.constants.Nanostar.VALUE` shares of Telegram
            Stars transferred by the transaction; from 0 to
            :tg-const:`~telegram.constants.NanostarLimit.MAX_AMOUNT`

            .. versionadded:: 21.9
        date (:obj:`datetime.datetime`): Date the transaction was created as a datetime object.
        source (:class:`telegram.TransactionPartner`, optional): Source of an incoming transaction
            (e.g., a user purchasing goods or services, Fragment refunding a failed withdrawal).
            Only for incoming transactions.
        receiver (:class:`telegram.TransactionPartner`, optional): Receiver of an outgoing
            transaction (e.g., a user for a purchase refund, Fragment for a withdrawal). Only for
            outgoing transactions.

    Attributes:
        id (:obj:`str`): Unique identifier of the transaction. Coincides with the identifer
            of the original transaction for refund transactions.
            Coincides with :attr:`SuccessfulPayment.telegram_payment_charge_id` for
            successful incoming payments from users.
        amount (:obj:`int`): Integer amount of Telegram Stars transferred by the transaction.
        nanostar_amount (:obj:`int`): Optional. The number of
            :tg-const:`~telegram.constants.Nanostar.VALUE` shares of Telegram
            Stars transferred by the transaction; from 0 to
            :tg-const:`~telegram.constants.NanostarLimit.MAX_AMOUNT`

            .. versionadded:: 21.9
        date (:obj:`datetime.datetime`): Date the transaction was created as a datetime object.
        source (:class:`telegram.TransactionPartner`): Optional. Source of an incoming transaction
            (e.g., a user purchasing goods or services, Fragment refunding a failed withdrawal).
            Only for incoming transactions.
        receiver (:class:`telegram.TransactionPartner`): Optional. Receiver of an outgoing
            transaction (e.g., a user for a purchase refund, Fragment for a withdrawal). Only for
            outgoing transactions.
    """

    __slots__ = ("amount", "date", "id", "nanostar_amount", "receiver", "source")

    def __init__(
        self,
        id: str,
        amount: int,
        date: dtm.datetime,
        source: TransactionPartner | None = None,
        receiver: TransactionPartner | None = None,
        nanostar_amount: int | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ) -> None:
        super().__init__(api_kwargs=api_kwargs)
        self.id: str = id
        self.amount: int = amount
        self.date: dtm.datetime = date
        self.source: TransactionPartner | None = source
        self.receiver: TransactionPartner | None = receiver
        self.nanostar_amount: int | None = nanostar_amount

        self._id_attrs = (
            self.id,
            self.source,
            self.receiver,
        )
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "StarTransaction":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)
        data["date"] = from_timestamp(data.get("date", None), tzinfo=loc_tzinfo)

        data["source"] = de_json_optional(data.get("source"), TransactionPartner, bot)
        data["receiver"] = de_json_optional(data.get("receiver"), TransactionPartner, bot)

        return super().de_json(data=data, bot=bot)


class StarTransactions(TelegramObject):
    """
    Contains a list of Telegram Star transactions.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`transactions` are equal.

    .. versionadded:: 21.4

    Args:
        transactions (Sequence[:class:`telegram.StarTransaction`]): The list of transactions.

    Attributes:
        transactions (tuple[:class:`telegram.StarTransaction`]): The list of transactions.
    """

    __slots__ = ("transactions",)

    def __init__(
        self, transactions: Sequence[StarTransaction], *, api_kwargs: JSONDict | None = None
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.transactions: tuple[StarTransaction, ...] = parse_sequence_arg(transactions)

        self._id_attrs = (self.transactions,)
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "StarTransactions":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["transactions"] = de_list_optional(data.get("transactions"), StarTransaction, bot)
        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains helper functions related to inspecting the program stack.

.. versionadded:: 20.0

Warning:
    Contents of this module are intended to be used internally by the library and *not* by the
    user. Changes to this module are not considered breaking changes and may not be documented in
    the changelog.
"""

from pathlib import Path
from types import FrameType

from telegram._utils.logging import get_logger

_LOGGER = get_logger(__name__)


def was_called_by(frame: FrameType | None, caller: Path) -> bool:
    """Checks if the passed frame was called by the specified file.

    Example:
        .. code:: pycon

            >>> was_called_by(inspect.currentframe(), Path(__file__))
            True

    Arguments:
        frame (:obj:`FrameType`): The frame - usually the return value of
            ``inspect.currentframe()``. If :obj:`None` is passed, the return value will be
            :obj:`False`.
        caller (:obj:`pathlib.Path`): File that should be the caller.

    Returns:
        :obj:`bool`: Whether the frame was called by the specified file.
    """
    if frame is None:
        return False

    try:
        return _was_called_by(frame, caller)
    except Exception as exc:
        _LOGGER.debug(
            "Failed to check if frame was called by `caller`. Assuming that it was not.",
            exc_info=exc,
        )
        return False


def _was_called_by(frame: FrameType, caller: Path) -> bool:
    # https://stackoverflow.com/a/57712700/10606962
    if Path(frame.f_code.co_filename).resolve() == caller:
        return True
    while frame.f_back:
        frame = frame.f_back
        if Path(frame.f_code.co_filename).resolve() == caller:
            return True
    return False



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program. If not, see [http://www.gnu.org/licenses/].


import pytest

from telegram._dice import Dice
from telegram._reaction import ReactionTypeEmoji
from telegram._storyarea import (
    LocationAddress,
    StoryArea,
    StoryAreaPosition,
    StoryAreaType,
    StoryAreaTypeLink,
    StoryAreaTypeLocation,
    StoryAreaTypeSuggestedReaction,
    StoryAreaTypeUniqueGift,
    StoryAreaTypeWeather,
)
from telegram.constants import StoryAreaTypeType
from tests.auxil.slots import mro_slots


@pytest.fixture
def story_area_position():
    return StoryAreaPosition(
        x_percentage=StoryAreaPositionTestBase.x_percentage,
        y_percentage=StoryAreaPositionTestBase.y_percentage,
        width_percentage=StoryAreaPositionTestBase.width_percentage,
        height_percentage=StoryAreaPositionTestBase.height_percentage,
        rotation_angle=StoryAreaPositionTestBase.rotation_angle,
        corner_radius_percentage=StoryAreaPositionTestBase.corner_radius_percentage,
    )


class StoryAreaPositionTestBase:
    x_percentage = 50.0
    y_percentage = 10.0
    width_percentage = 15
    height_percentage = 15
    rotation_angle = 0.0
    corner_radius_percentage = 8.0


class TestStoryAreaPositionWithoutRequest(StoryAreaPositionTestBase):
    def test_slot_behaviour(self, story_area_position):
        inst = story_area_position
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, story_area_position):
        assert story_area_position.x_percentage == self.x_percentage
        assert story_area_position.y_percentage == self.y_percentage
        assert story_area_position.width_percentage == self.width_percentage
        assert story_area_position.height_percentage == self.height_percentage
        assert story_area_position.rotation_angle == self.rotation_angle
        assert story_area_position.corner_radius_percentage == self.corner_radius_percentage

    def test_to_dict(self, story_area_position):
        json_dict = story_area_position.to_dict()
        assert json_dict["x_percentage"] == self.x_percentage
        assert json_dict["y_percentage"] == self.y_percentage
        assert json_dict["width_percentage"] == self.width_percentage
        assert json_dict["height_percentage"] == self.height_percentage
        assert json_dict["rotation_angle"] == self.rotation_angle
        assert json_dict["corner_radius_percentage"] == self.corner_radius_percentage

    def test_equality(self, story_area_position):
        a = story_area_position
        b = StoryAreaPosition(
            self.x_percentage,
            self.y_percentage,
            self.width_percentage,
            self.height_percentage,
            self.rotation_angle,
            self.corner_radius_percentage,
        )
        c = StoryAreaPosition(
            self.x_percentage + 10.0,
            self.y_percentage,
            self.width_percentage,
            self.height_percentage,
            self.rotation_angle,
            self.corner_radius_percentage,
        )
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def location_address():
    return LocationAddress(
        country_code=LocationAddressTestBase.country_code,
        state=LocationAddressTestBase.state,
        city=LocationAddressTestBase.city,
        street=LocationAddressTestBase.street,
    )


class LocationAddressTestBase:
    country_code = "CC"
    state = "State"
    city = "City"
    street = "12 downtown"


class TestLocationAddressWithoutRequest(LocationAddressTestBase):
    def test_slot_behaviour(self, location_address):
        inst = location_address
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, location_address):
        assert location_address.country_code == self.country_code
        assert location_address.state == self.state
        assert location_address.city == self.city
        assert location_address.street == self.street

    def test_to_dict(self, location_address):
        json_dict = location_address.to_dict()
        assert json_dict["country_code"] == self.country_code
        assert json_dict["state"] == self.state
        assert json_dict["city"] == self.city
        assert json_dict["street"] == self.street

    def test_equality(self, location_address):
        a = location_address
        b = LocationAddress(self.country_code, self.state, self.city, self.street)
        c = LocationAddress("some_other_code", self.state, self.city, self.street)
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def story_area():
    return StoryArea(
        position=StoryAreaTestBase.position,
        type=StoryAreaTestBase.type,
    )


class StoryAreaTestBase:
    position = StoryAreaPosition(
        x_percentage=50.0,
        y_percentage=10.0,
        width_percentage=15,
        height_percentage=15,
        rotation_angle=0.0,
        corner_radius_percentage=8.0,
    )
    type = StoryAreaTypeLink(url="some_url")


class TestStoryAreaWithoutRequest(StoryAreaTestBase):
    def test_slot_behaviour(self, story_area):
        inst = story_area
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, story_area):
        assert story_area.position == self.position
        assert story_area.type == self.type

    def test_to_dict(self, story_area):
        json_dict = story_area.to_dict()
        assert json_dict["position"] == self.position.to_dict()
        assert json_dict["type"] == self.type.to_dict()

    def test_equality(self, story_area):
        a = story_area
        b = StoryArea(self.position, self.type)
        c = StoryArea(self.position, StoryAreaTypeLink(url="some_other_url"))
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def story_area_type():
    return StoryAreaType(type=StoryAreaTypeTestBase.type)


class StoryAreaTypeTestBase:
    type = StoryAreaTypeType.LOCATION
    latitude = 100.5
    longitude = 200.5
    address = LocationAddress(
        country_code="cc",
        state="State",
        city="City",
        street="12 downtown",
    )
    reaction_type = ReactionTypeEmoji(emoji="emoji")
    is_dark = False
    is_flipped = False
    url = "http_url"
    temperature = 35.0
    emoji = "emoji"
    background_color = 0xFF66CCFF
    name = "unique_gift_name"


class TestStoryAreaTypeWithoutRequest(StoryAreaTypeTestBase):
    def test_slot_behaviour(self, story_area_type):
        inst = story_area_type
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, story_area_type):
        assert story_area_type.type == self.type

    def test_type_enum_conversion(self, story_area_type):
        assert type(StoryAreaType("location").type) is StoryAreaTypeType
        assert StoryAreaType("unknown").type == "unknown"

    def test_to_dict(self, story_area_type):
        assert story_area_type.to_dict() == {"type": self.type}

    def test_equality(self, story_area_type):
        a = story_area_type
        b = StoryAreaType(self.type)
        c = StoryAreaType("unknown")
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def story_area_type_location():
    return StoryAreaTypeLocation(
        latitude=TestStoryAreaTypeLocationWithoutRequest.latitude,
        longitude=TestStoryAreaTypeLocationWithoutRequest.longitude,
        address=TestStoryAreaTypeLocationWithoutRequest.address,
    )


class TestStoryAreaTypeLocationWithoutRequest(StoryAreaTypeTestBase):
    type = StoryAreaTypeType.LOCATION

    def test_slot_behaviour(self, story_area_type_location):
        inst = story_area_type_location
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, story_area_type_location):
        assert story_area_type_location.type == self.type
        assert story_area_type_location.latitude == self.latitude
        assert story_area_type_location.longitude == self.longitude
        assert story_area_type_location.address == self.address

    def test_to_dict(self, story_area_type_location):
        json_dict = story_area_type_location.to_dict()
        assert isinstance(json_dict, dict)
        assert json_dict["type"] == self.type
        assert json_dict["latitude"] == self.latitude
        assert json_dict["longitude"] == self.longitude
        assert json_dict["address"] == self.address.to_dict()

    def test_equality(self, story_area_type_location):
        a = story_area_type_location
        b = StoryAreaTypeLocation(self.latitude, self.longitude, self.address)
        c = StoryAreaTypeLocation(self.latitude + 0.5, self.longitude, self.address)
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def story_area_type_suggested_reaction():
    return StoryAreaTypeSuggestedReaction(
        reaction_type=TestStoryAreaTypeSuggestedReactionWithoutRequest.reaction_type,
        is_dark=TestStoryAreaTypeSuggestedReactionWithoutRequest.is_dark,
        is_flipped=TestStoryAreaTypeSuggestedReactionWithoutRequest.is_flipped,
    )


class TestStoryAreaTypeSuggestedReactionWithoutRequest(StoryAreaTypeTestBase):
    type = StoryAreaTypeType.SUGGESTED_REACTION

    def test_slot_behaviour(self, story_area_type_suggested_reaction):
        inst = story_area_type_suggested_reaction
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, story_area_type_suggested_reaction):
        assert story_area_type_suggested_reaction.type == self.type
        assert story_area_type_suggested_reaction.reaction_type == self.reaction_type
        assert story_area_type_suggested_reaction.is_dark is self.is_dark
        assert story_area_type_suggested_reaction.is_flipped is self.is_flipped

    def test_to_dict(self, story_area_type_suggested_reaction):
        json_dict = story_area_type_suggested_reaction.to_dict()
        assert isinstance(json_dict, dict)
        assert json_dict["type"] == self.type
        assert json_dict["reaction_type"] == self.reaction_type.to_dict()
        assert json_dict["is_dark"] is self.is_dark
        assert json_dict["is_flipped"] is self.is_flipped

    def test_equality(self, story_area_type_suggested_reaction):
        a = story_area_type_suggested_reaction
        b = StoryAreaTypeSuggestedReaction(self.reaction_type, self.is_dark, self.is_flipped)
        c = StoryAreaTypeSuggestedReaction(self.reaction_type, not self.is_dark, self.is_flipped)
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def story_area_type_link():
    return StoryAreaTypeLink(
        url=TestStoryAreaTypeLinkWithoutRequest.url,
    )


class TestStoryAreaTypeLinkWithoutRequest(StoryAreaTypeTestBase):
    type = StoryAreaTypeType.LINK

    def test_slot_behaviour(self, story_area_type_link):
        inst = story_area_type_link
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, story_area_type_link):
        assert story_area_type_link.type == self.type
        assert story_area_type_link.url == self.url

    def test_to_dict(self, story_area_type_link):
        json_dict = story_area_type_link.to_dict()
        assert isinstance(json_dict, dict)
        assert json_dict["type"] == self.type
        assert json_dict["url"] == self.url

    def test_equality(self, story_area_type_link):
        a = story_area_type_link
        b = StoryAreaTypeLink(self.url)
        c = StoryAreaTypeLink("other_http_url")
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def story_area_type_weather():
    return StoryAreaTypeWeather(
        temperature=TestStoryAreaTypeWeatherWithoutRequest.temperature,
        emoji=TestStoryAreaTypeWeatherWithoutRequest.emoji,
        background_color=TestStoryAreaTypeWeatherWithoutRequest.background_color,
    )


class TestStoryAreaTypeWeatherWithoutRequest(StoryAreaTypeTestBase):
    type = StoryAreaTypeType.WEATHER

    def test_slot_behaviour(self, story_area_type_weather):
        inst = story_area_type_weather
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, story_area_type_weather):
        assert story_area_type_weather.type == self.type
        assert story_area_type_weather.temperature == self.temperature
        assert story_area_type_weather.emoji == self.emoji
        assert story_area_type_weather.background_color == self.background_color

    def test_to_dict(self, story_area_type_weather):
        json_dict = story_area_type_weather.to_dict()
        assert isinstance(json_dict, dict)
        assert json_dict["type"] == self.type
        assert json_dict["temperature"] == self.temperature
        assert json_dict["emoji"] == self.emoji
        assert json_dict["background_color"] == self.background_color

    def test_equality(self, story_area_type_weather):
        a = story_area_type_weather
        b = StoryAreaTypeWeather(self.temperature, self.emoji, self.background_color)
        c = StoryAreaTypeWeather(self.temperature - 5.0, self.emoji, self.background_color)
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def story_area_type_unique_gift():
    return StoryAreaTypeUniqueGift(
        name=TestStoryAreaTypeUniqueGiftWithoutRequest.name,
    )


class TestStoryAreaTypeUniqueGiftWithoutRequest(StoryAreaTypeTestBase):
    type = StoryAreaTypeType.UNIQUE_GIFT

    def test_slot_behaviour(self, story_area_type_unique_gift):
        inst = story_area_type_unique_gift
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, story_area_type_unique_gift):
        assert story_area_type_unique_gift.type == self.type
        assert story_area_type_unique_gift.name == self.name

    def test_to_dict(self, story_area_type_unique_gift):
        json_dict = story_area_type_unique_gift.to_dict()
        assert isinstance(json_dict, dict)
        assert json_dict["type"] == self.type
        assert json_dict["name"] == self.name

    def test_equality(self, story_area_type_unique_gift):
        a = story_area_type_unique_gift
        b = StoryAreaTypeUniqueGift(self.name)
        c = StoryAreaTypeUniqueGift("other_name")
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""Contains information about Telegram Passport data shared with the bot by the user."""

from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._passport.credentials import EncryptedCredentials
from telegram._passport.encryptedpassportelement import EncryptedPassportElement
from telegram._telegramobject import TelegramObject
from telegram._utils.argumentparsing import de_json_optional, de_list_optional, parse_sequence_arg
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot, Credentials


class PassportData(TelegramObject):
    """Contains information about Telegram Passport data shared with the bot by the user.

    Note:
        To be able to decrypt this object, you must pass your ``private_key`` to either
        :class:`telegram.ext.Updater` or :class:`telegram.Bot`. Decrypted data is then found in
        :attr:`decrypted_data` and the payload can be found in :attr:`decrypted_credentials`'s
        attribute :attr:`telegram.Credentials.nonce`.

    Args:
        data (Sequence[:class:`telegram.EncryptedPassportElement`]): Array with encrypted
            information about documents and other Telegram Passport elements that was shared with
            the bot.

            .. versionchanged:: 20.0
                |sequenceclassargs|

        credentials (:class:`telegram.EncryptedCredentials`)): Encrypted credentials.

    Attributes:
        data (tuple[:class:`telegram.EncryptedPassportElement`]): Array with encrypted
            information about documents and other Telegram Passport elements that was shared with
            the bot.

            .. versionchanged:: 20.0
                |tupleclassattrs|

        credentials (:class:`telegram.EncryptedCredentials`): Encrypted credentials.


    """

    __slots__ = ("_decrypted_data", "credentials", "data")

    def __init__(
        self,
        data: Sequence[EncryptedPassportElement],
        credentials: EncryptedCredentials,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        self.data: tuple[EncryptedPassportElement, ...] = parse_sequence_arg(data)
        self.credentials: EncryptedCredentials = credentials

        self._decrypted_data: tuple[EncryptedPassportElement] | None = None
        self._id_attrs = tuple([x.type for x in data] + [credentials.hash])

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "PassportData":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["data"] = de_list_optional(data.get("data"), EncryptedPassportElement, bot)
        data["credentials"] = de_json_optional(data.get("credentials"), EncryptedCredentials, bot)

        return super().de_json(data=data, bot=bot)

    @property
    def decrypted_data(self) -> tuple[EncryptedPassportElement, ...]:
        """
        tuple[:class:`telegram.EncryptedPassportElement`]: Lazily decrypt and return information
            about documents and other Telegram Passport elements which were shared with the bot.

        .. versionchanged:: 20.0
            Returns a tuple instead of a list.

        Raises:
            telegram.error.PassportDecryptionError: Decryption failed. Usually due to bad
                private/public key but can also suggest malformed/tampered data.
        """
        if self._decrypted_data is None:
            self._decrypted_data = tuple(  # type: ignore[assignment]
                EncryptedPassportElement.de_json_decrypted(
                    element.to_dict(), self.get_bot(), self.decrypted_credentials
                )
                for element in self.data
            )
        return self._decrypted_data  # type: ignore[return-value]

    @property
    def decrypted_credentials(self) -> "Credentials":
        """
        :class:`telegram.Credentials`: Lazily decrypt and return credentials that were used
            to decrypt the data. This object also contains the user specified payload as
            `decrypted_data.payload`.

        Raises:
            telegram.error.PassportDecryptionError: Decryption failed. Usually due to bad
                private/public key but can also suggest malformed/tampered data.
        """
        return self.credentials.decrypted_data



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

"""
This module is intentionally named without "test_" prefix.
These tests are supposed to be run on GitHub when building docs.
The tests require Python 3.10+ (just like AdmonitionInserter being tested),
so they cannot be included in the main suite while older versions of Python are supported.
"""

import collections.abc

import pytest

import telegram.ext
from docs.auxil.admonition_inserter import AdmonitionInserter


@pytest.fixture(scope="session")
def admonition_inserter():
    return AdmonitionInserter()


class TestAdmonitionInserter:
    """This is a minimal-effort test to ensure that the `AdmonitionInserter`
    used for automatically inserting references in the docs works as expected.

    It does not aim to cover all links in the documentation, but rather checks that several special
    cases (which where discovered during the implementation of `AdmonitionInserter`) are handled
    correctly.
    """

    def test_admonitions_dict(self, admonition_inserter):
        # there are keys for every type of admonition
        assert len(admonition_inserter.admonitions) == len(
            admonition_inserter.ALL_ADMONITION_TYPES
        )

        # for each type of admonitions, there is at least one entry
        # ({class/method: admonition text})
        for admonition_type in admonition_inserter.ALL_ADMONITION_TYPES:
            assert admonition_type in admonition_inserter.admonitions
            assert len(admonition_inserter.admonitions[admonition_type].keys()) > 0

        # checking class admonitions
        for admonition_type in admonition_inserter.CLASS_ADMONITION_TYPES:
            # keys are telegram classes
            for cls in admonition_inserter.admonitions[admonition_type]:
                # Test classes crop up in AppBuilder, they can't come from code being tested.
                if "tests." in str(cls):
                    continue

                assert isinstance(cls, type)
                assert str(cls).startswith("<class 'telegram."), (
                    rf"Class {cls} does not belong to Telegram classes. Admonition:\n"
                    rf"{admonition_inserter.admonitions[admonition_type][cls]}"
                )

        # checking Bot method admonitions
        for admonition_type in admonition_inserter.METHOD_ADMONITION_TYPES:
            for method in admonition_inserter.admonitions[admonition_type]:
                assert isinstance(method, collections.abc.Callable)
                assert str(method).startswith("<function Bot."), (
                    f"Method {method} does not belong to methods that should get admonitions."
                    "Admonition:\n"
                    f"{admonition_inserter.admonitions[admonition_type][method]}"
                )

    @pytest.mark.parametrize(
        ("admonition_type", "cls", "link"),
        [
            (
                "available_in",
                telegram.ChatMember,
                ":attr:`telegram.ChatMemberUpdated.new_chat_member`",
            ),
            (
                "available_in",
                telegram.ChatMemberAdministrator,
                ":attr:`telegram.ChatMemberUpdated.new_chat_member`",
            ),
            (
                "available_in",
                telegram.Sticker,
                ":attr:`telegram.StickerSet.stickers`",  # tuple[telegram.Sticker]
            ),
            (
                "available_in",
                telegram.ResidentialAddress,  # mentioned on the second line of docstring of .data
                ":attr:`telegram.EncryptedPassportElement.data`",
            ),
            (
                "available_in",
                telegram.ext.JobQueue,
                ":attr:`telegram.ext.CallbackContext.job_queue`",
            ),
            (
                "available_in",
                telegram.ext.Application,
                ":attr:`telegram.ext.CallbackContext.application`",
            ),
            (
                "available_in",
                telegram.Bot,
                ":attr:`telegram.ext.CallbackContext.bot`",
            ),
            (
                "available_in",
                telegram.Bot,
                ":attr:`telegram.ext.Application.bot`",
            ),
            (
                "returned_in",
                telegram.StickerSet,
                ":meth:`telegram.Bot.get_sticker_set`",
            ),
            (
                "returned_in",
                telegram.ChatMember,
                ":meth:`telegram.Bot.get_chat_member`",
            ),
            (
                "returned_in",
                telegram.GameHighScore,
                ":meth:`telegram.Bot.get_game_high_scores`",
            ),
            (
                "returned_in",
                telegram.ChatMemberOwner,
                ":meth:`telegram.Bot.get_chat_member`",  # subclass
            ),
            (
                "returned_in",
                telegram.Message,
                ":meth:`telegram.Bot.edit_message_live_location`",  # Union[Message, bool]
            ),
            (
                "returned_in",
                telegram.ext.Application,
                ":meth:`telegram.ext.ApplicationBuilder.build`",  # <class 'types.GenericAlias'>
            ),
            (
                "shortcuts",
                telegram.Bot.edit_message_caption,
                # this method in CallbackQuery contains two return statements,
                # one of which is with Bot
                ":meth:`telegram.CallbackQuery.edit_message_caption`",
            ),
            (
                "shortcuts",
                telegram.Bot.ban_chat_member,
                # ban_member is defined on the private parent class _ChatBase
                ":meth:`telegram.Chat.ban_member`",
            ),
            (
                "shortcuts",
                telegram.Bot.ban_chat_member,
                # ban_member is defined on the private parent class _ChatBase
                ":meth:`telegram.ChatFullInfo.ban_member`",
            ),
            (
                "use_in",
                telegram.InlineQueryResult,
                ":meth:`telegram.Bot.answer_web_app_query`",  # ForwardRef
            ),
            (
                "use_in",
                telegram.InputMediaPhoto,
                ":meth:`telegram.Bot.send_media_group`",  # Sequence[Union[...]]
            ),
            (
                "use_in",
                telegram.InlineKeyboardMarkup,
                ":meth:`telegram.Bot.send_message`",  # optional
            ),
            (
                "use_in",
                telegram.Sticker,
                ":meth:`telegram.Bot.get_file`",  # .file_id with lots of piped types
            ),
            (
                "use_in",
                telegram.ext.BasePersistence,
                ":meth:`telegram.ext.ApplicationBuilder.persistence`",
            ),
            ("use_in", telegram.ext.Defaults, ":meth:`telegram.ext.ApplicationBuilder.defaults`"),
            (
                "use_in",
                telegram.ext.JobQueue,
                ":meth:`telegram.ext.ApplicationBuilder.job_queue`",  # TypeVar
            ),
            (
                "use_in",
                telegram.ext.PicklePersistence,  # subclass
                ":meth:`telegram.ext.ApplicationBuilder.persistence`",
            ),
        ],
    )
    def test_check_presence(self, admonition_inserter, admonition_type, cls, link):
        """Checks if a given link is present in the admonition of a given type for a given
        class.
        """
        admonitions = admonition_inserter.admonitions

        assert cls in admonitions[admonition_type]

        # exactly one of the lines in the admonition for this class must consist of the link
        # (this is a stricter check than just checking if the entire admonition contains the link)
        lines_with_link = [
            line
            for line in admonitions[admonition_type][cls].splitlines()
            # remove whitespaces and occasional bullet list marker
            if line.strip().removeprefix("* ") == link
        ]
        assert lines_with_link, (
            f"Class {cls}, does not have link {link} in a {admonition_type} admonition:\n"
            f"{admonitions[admonition_type][cls]}"
        )
        assert len(lines_with_link) == 1, (
            f"Class {cls}, must contain only one link {link} in a {admonition_type} admonition:\n"
            f"{admonitions[admonition_type][cls]}"
        )

    @pytest.mark.parametrize(
        ("admonition_type", "cls", "link"),
        [
            (
                "returned_in",
                telegram.ext.CallbackContext,
                # -> Application[BT, CCT, UD, CD, BD, JQ].
                # The type vars are not really part of the return value, so we don't expect them
                ":meth:`telegram.ext.ApplicationBuilder.build`",
            ),
            (
                "returned_in",
                telegram.Bot,
                # -> Application[BT, CCT, UD, CD, BD, JQ].
                # The type vars are not really part of the return value, so we don't expect them
                ":meth:`telegram.ext.ApplicationBuilder.bot`",
            ),
        ],
    )
    def test_check_absence(self, admonition_inserter, admonition_type, cls, link):
        """Checks if a given link is **absent** in the admonition of a given type for a given
        class.

        If a given class has no admonition of this type at all, the test will also pass.
        """
        admonitions = admonition_inserter.admonitions

        assert not (
            cls in admonitions[admonition_type]
            and [
                line
                for line in admonitions[admonition_type][cls].splitlines()
                # remove whitespaces and occasional bullet list marker
                if line.strip().removeprefix("* ") == link
            ]
        ), (
            f"Class {cls} is not supposed to have link {link} in a {admonition_type} admonition:\n"
            f"{admonitions[admonition_type][cls]}"
        )
